#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_vpn_ipsec_phase1
short_description: Configure VPN remote gateway in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify vpn_ipsec feature and phase1 category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    vpn_ipsec_phase1:
        description:
            - Configure VPN remote gateway.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            acct-verify:
                description:
                    - Enable/disable verification of RADIUS accounting record.
                choices:
                    - enable
                    - disable
            add-gw-route:
                description:
                    - Enable/disable automatically add a route to the remote gateway.
                choices:
                    - enable
                    - disable
            add-route:
                description:
                    - Enable/disable control addition of a route to peer destination selector.
                choices:
                    - disable
                    - enable
            assign-ip:
                description:
                    - Enable/disable assignment of IP to IPsec interface via configuration method.
                choices:
                    - disable
                    - enable
            assign-ip-from:
                description:
                    - Method by which the IP address will be assigned.
                choices:
                    - range
                    - usrgrp
                    - dhcp
                    - name
            authmethod:
                description:
                    - Authentication method.
                choices:
                    - psk
                    - signature
            authmethod-remote:
                description:
                    - Authentication method (remote side).
                choices:
                    - psk
                    - signature
            authpasswd:
                description:
                    - XAuth password (max 35 characters).
            authusr:
                description:
                    - XAuth user name.
            authusrgrp:
                description:
                    - Authentication user group. Source user.group.name.
            auto-negotiate:
                description:
                    - Enable/disable automatic initiation of IKE SA negotiation.
                choices:
                    - enable
                    - disable
            backup-gateway:
                description:
                    - Instruct unity clients about the backup gateway address(es).
                suboptions:
                    address:
                        description:
                            - Address of backup gateway.
                        required: true
            banner:
                description:
                    - Message that unity client should display after connecting.
            cert-id-validation:
                description:
                    - Enable/disable cross validation of peer ID and the identity in the peer's certificate as specified in RFC 4945.
                choices:
                    - enable
                    - disable
            certificate:
                description:
                    - Names of up to 4 signed personal certificates.
                suboptions:
                    name:
                        description:
                            - Certificate name. Source vpn.certificate.local.name.
                        required: true
            childless-ike:
                description:
                    - Enable/disable childless IKEv2 initiation (RFC 6023).
                choices:
                    - enable
                    - disable
            client-auto-negotiate:
                description:
                    - Enable/disable allowing the VPN client to bring up the tunnel when there is no traffic.
                choices:
                    - disable
                    - enable
            client-keep-alive:
                description:
                    - Enable/disable allowing the VPN client to keep the tunnel up when there is no traffic.
                choices:
                    - disable
                    - enable
            comments:
                description:
                    - Comment.
            dhgrp:
                description:
                    - DH group.
                choices:
                    - 1
                    - 2
                    - 5
                    - 14
                    - 15
                    - 16
                    - 17
                    - 18
                    - 19
                    - 20
                    - 21
                    - 27
                    - 28
                    - 29
                    - 30
                    - 31
            digital-signature-auth:
                description:
                    - Enable/disable IKEv2 Digital Signature Authentication (RFC 7427).
                choices:
                    - enable
                    - disable
            distance:
                description:
                    - Distance for routes added by IKE (1 - 255).
            dns-mode:
                description:
                    - DNS server mode.
                choices:
                    - manual
                    - auto
            domain:
                description:
                    - Instruct unity clients about the default DNS domain.
            dpd:
                description:
                    - Dead Peer Detection mode.
                choices:
                    - disable
                    - on-idle
                    - on-demand
            dpd-retrycount:
                description:
                    - Number of DPD retry attempts.
            dpd-retryinterval:
                description:
                    - DPD retry interval.
            eap:
                description:
                    - Enable/disable IKEv2 EAP authentication.
                choices:
                    - enable
                    - disable
            eap-identity:
                description:
                    - IKEv2 EAP peer identity type.
                choices:
                    - use-id-payload
                    - send-request
            enforce-unique-id:
                description:
                    - Enable/disable peer ID uniqueness check.
                choices:
                    - disable
                    - keep-new
                    - keep-old
            forticlient-enforcement:
                description:
                    - Enable/disable FortiClient enforcement.
                choices:
                    - enable
                    - disable
            fragmentation:
                description:
                    - Enable/disable fragment IKE message on re-transmission.
                choices:
                    - enable
                    - disable
            fragmentation-mtu:
                description:
                    - IKE fragmentation MTU (500 - 16000).
            group-authentication:
                description:
                    - Enable/disable IKEv2 IDi group authentication.
                choices:
                    - enable
                    - disable
            group-authentication-secret:
                description:
                    - Password for IKEv2 IDi group authentication.  (ASCII string or hexadecimal indicated by a leading 0x.)
            ha-sync-esp-seqno:
                description:
                    - Enable/disable sequence number jump ahead for IPsec HA.
                choices:
                    - enable
                    - disable
            idle-timeout:
                description:
                    - Enable/disable IPsec tunnel idle timeout.
                choices:
                    - enable
                    - disable
            idle-timeoutinterval:
                description:
                    - IPsec tunnel idle timeout in minutes (5 - 43200).
            ike-version:
                description:
                    - IKE protocol version.
                choices:
                    - 1
                    - 2
            include-local-lan:
                description:
                    - Enable/disable allow local LAN access on unity clients.
                choices:
                    - disable
                    - enable
            interface:
                description:
                    - Local physical, aggregate, or VLAN outgoing interface. Source system.interface.name.
            ipv4-dns-server1:
                description:
                    - IPv4 DNS server 1.
            ipv4-dns-server2:
                description:
                    - IPv4 DNS server 2.
            ipv4-dns-server3:
                description:
                    - IPv4 DNS server 3.
            ipv4-end-ip:
                description:
                    - End of IPv4 range.
            ipv4-exclude-range:
                description:
                    - Configuration Method IPv4 exclude ranges.
                suboptions:
                    end-ip:
                        description:
                            - End of IPv4 exclusive range.
                    id:
                        description:
                            - ID.
                        required: true
                    start-ip:
                        description:
                            - Start of IPv4 exclusive range.
            ipv4-name:
                description:
                    - IPv4 address name. Source firewall.address.name firewall.addrgrp.name.
            ipv4-netmask:
                description:
                    - IPv4 Netmask.
            ipv4-split-exclude:
                description:
                    - IPv4 subnets that should not be sent over the IPsec tunnel. Source firewall.address.name firewall.addrgrp.name.
            ipv4-split-include:
                description:
                    - IPv4 split-include subnets. Source firewall.address.name firewall.addrgrp.name.
            ipv4-start-ip:
                description:
                    - Start of IPv4 range.
            ipv4-wins-server1:
                description:
                    - WINS server 1.
            ipv4-wins-server2:
                description:
                    - WINS server 2.
            ipv6-dns-server1:
                description:
                    - IPv6 DNS server 1.
            ipv6-dns-server2:
                description:
                    - IPv6 DNS server 2.
            ipv6-dns-server3:
                description:
                    - IPv6 DNS server 3.
            ipv6-end-ip:
                description:
                    - End of IPv6 range.
            ipv6-exclude-range:
                description:
                    - Configuration method IPv6 exclude ranges.
                suboptions:
                    end-ip:
                        description:
                            - End of IPv6 exclusive range.
                    id:
                        description:
                            - ID.
                        required: true
                    start-ip:
                        description:
                            - Start of IPv6 exclusive range.
            ipv6-name:
                description:
                    - IPv6 address name. Source firewall.address6.name firewall.addrgrp6.name.
            ipv6-prefix:
                description:
                    - IPv6 prefix.
            ipv6-split-exclude:
                description:
                    - IPv6 subnets that should not be sent over the IPsec tunnel. Source firewall.address6.name firewall.addrgrp6.name.
            ipv6-split-include:
                description:
                    - IPv6 split-include subnets. Source firewall.address6.name firewall.addrgrp6.name.
            ipv6-start-ip:
                description:
                    - Start of IPv6 range.
            keepalive:
                description:
                    - NAT-T keep alive interval.
            keylife:
                description:
                    - Time to wait in seconds before phase 1 encryption key expires.
            local-gw:
                description:
                    - Local VPN gateway.
            localid:
                description:
                    - Local ID.
            localid-type:
                description:
                    - Local ID type.
                choices:
                    - auto
                    - fqdn
                    - user-fqdn
                    - keyid
                    - address
                    - asn1dn
            mesh-selector-type:
                description:
                    - Add selectors containing subsets of the configuration depending on traffic.
                choices:
                    - disable
                    - subnet
                    - host
            mode:
                description:
                    - ID protection mode used to establish a secure channel.
                choices:
                    - aggressive
                    - main
            mode-cfg:
                description:
                    - Enable/disable configuration method.
                choices:
                    - disable
                    - enable
            name:
                description:
                    - IPsec remote gateway name.
                required: true
            nattraversal:
                description:
                    - Enable/disable NAT traversal.
                choices:
                    - enable
                    - disable
                    - forced
            negotiate-timeout:
                description:
                    - IKE SA negotiation timeout in seconds (1 - 300).
            npu-offload:
                description:
                    - Enable/disable offloading NPU.
                choices:
                    - enable
                    - disable
            peer:
                description:
                    - Accept this peer certificate. Source user.peer.name.
            peergrp:
                description:
                    - Accept this peer certificate group. Source user.peergrp.name.
            peerid:
                description:
                    - Accept this peer identity.
            peertype:
                description:
                    - Accept this peer type.
                choices:
                    - any
                    - one
                    - dialup
                    - peer
                    - peergrp
            ppk:
                description:
                    - Enable/disable IKEv2 Postquantum Preshared Key (PPK).
                choices:
                    - disable
                    - allow
                    - require
            ppk-identity:
                description:
                    - IKEv2 Postquantum Preshared Key Identity.
            ppk-secret:
                description:
                    - IKEv2 Postquantum Preshared Key (ASCII string or hexadecimal encoded with a leading 0x).
            priority:
                description:
                    - Priority for routes added by IKE (0 - 4294967295).
            proposal:
                description:
                    - Phase1 proposal.
                choices:
                    - des-md5
                    - des-sha1
                    - des-sha256
                    - des-sha384
                    - des-sha512
            psksecret:
                description:
                    - Pre-shared secret for PSK authentication (ASCII string or hexadecimal encoded with a leading 0x).
            psksecret-remote:
                description:
                    - Pre-shared secret for remote side PSK authentication (ASCII string or hexadecimal encoded with a leading 0x).
            reauth:
                description:
                    - Enable/disable re-authentication upon IKE SA lifetime expiration.
                choices:
                    - disable
                    - enable
            rekey:
                description:
                    - Enable/disable phase1 rekey.
                choices:
                    - enable
                    - disable
            remote-gw:
                description:
                    - Remote VPN gateway.
            remotegw-ddns:
                description:
                    - Domain name of remote gateway (eg. name.DDNS.com).
            rsa-signature-format:
                description:
                    - Digital Signature Authentication RSA signature format.
                choices:
                    - pkcs1
                    - pss
            save-password:
                description:
                    - Enable/disable saving XAuth username and password on VPN clients.
                choices:
                    - disable
                    - enable
            send-cert-chain:
                description:
                    - Enable/disable sending certificate chain.
                choices:
                    - enable
                    - disable
            signature-hash-alg:
                description:
                    - Digital Signature Authentication hash algorithms.
                choices:
                    - sha1
                    - sha2-256
                    - sha2-384
                    - sha2-512
            split-include-service:
                description:
                    - Split-include services. Source firewall.service.group.name firewall.service.custom.name.
            suite-b:
                description:
                    - Use Suite-B.
                choices:
                    - disable
                    - suite-b-gcm-128
                    - suite-b-gcm-256
            type:
                description:
                    - Remote gateway type.
                choices:
                    - static
                    - dynamic
                    - ddns
            unity-support:
                description:
                    - Enable/disable support for Cisco UNITY Configuration Method extensions.
                choices:
                    - disable
                    - enable
            usrgrp:
                description:
                    - User group name for dialup peers. Source user.group.name.
            wizard-type:
                description:
                    - GUI VPN Wizard Type.
                choices:
                    - custom
                    - dialup-forticlient
                    - dialup-ios
                    - dialup-android
                    - dialup-windows
                    - dialup-cisco
                    - static-fortigate
                    - dialup-fortigate
                    - static-cisco
                    - dialup-cisco-fw
            xauthtype:
                description:
                    - XAuth type.
                choices:
                    - disable
                    - client
                    - pap
                    - chap
                    - auto
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure VPN remote gateway.
    fortios_vpn_ipsec_phase1:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      vpn_ipsec_phase1:
        state: "present"
        acct-verify: "enable"
        add-gw-route: "enable"
        add-route: "disable"
        assign-ip: "disable"
        assign-ip-from: "range"
        authmethod: "psk"
        authmethod-remote: "psk"
        authpasswd: "<your_own_value>"
        authusr: "<your_own_value>"
        authusrgrp: "<your_own_value> (source user.group.name)"
        auto-negotiate: "enable"
        backup-gateway:
         -
            address: "<your_own_value>"
        banner: "<your_own_value>"
        cert-id-validation: "enable"
        certificate:
         -
            name: "default_name_19 (source vpn.certificate.local.name)"
        childless-ike: "enable"
        client-auto-negotiate: "disable"
        client-keep-alive: "disable"
        comments: "<your_own_value>"
        dhgrp: "1"
        digital-signature-auth: "enable"
        distance: "26"
        dns-mode: "manual"
        domain: "<your_own_value>"
        dpd: "disable"
        dpd-retrycount: "30"
        dpd-retryinterval: "<your_own_value>"
        eap: "enable"
        eap-identity: "use-id-payload"
        enforce-unique-id: "disable"
        forticlient-enforcement: "enable"
        fragmentation: "enable"
        fragmentation-mtu: "37"
        group-authentication: "enable"
        group-authentication-secret: "<your_own_value>"
        ha-sync-esp-seqno: "enable"
        idle-timeout: "enable"
        idle-timeoutinterval: "42"
        ike-version: "1"
        include-local-lan: "disable"
        interface: "<your_own_value> (source system.interface.name)"
        ipv4-dns-server1: "<your_own_value>"
        ipv4-dns-server2: "<your_own_value>"
        ipv4-dns-server3: "<your_own_value>"
        ipv4-end-ip: "<your_own_value>"
        ipv4-exclude-range:
         -
            end-ip: "<your_own_value>"
            id:  "52"
            start-ip: "<your_own_value>"
        ipv4-name: "<your_own_value> (source firewall.address.name firewall.addrgrp.name)"
        ipv4-netmask: "<your_own_value>"
        ipv4-split-exclude: "<your_own_value> (source firewall.address.name firewall.addrgrp.name)"
        ipv4-split-include: "<your_own_value> (source firewall.address.name firewall.addrgrp.name)"
        ipv4-start-ip: "<your_own_value>"
        ipv4-wins-server1: "<your_own_value>"
        ipv4-wins-server2: "<your_own_value>"
        ipv6-dns-server1: "<your_own_value>"
        ipv6-dns-server2: "<your_own_value>"
        ipv6-dns-server3: "<your_own_value>"
        ipv6-end-ip: "<your_own_value>"
        ipv6-exclude-range:
         -
            end-ip: "<your_own_value>"
            id:  "67"
            start-ip: "<your_own_value>"
        ipv6-name: "<your_own_value> (source firewall.address6.name firewall.addrgrp6.name)"
        ipv6-prefix: "70"
        ipv6-split-exclude: "<your_own_value> (source firewall.address6.name firewall.addrgrp6.name)"
        ipv6-split-include: "<your_own_value> (source firewall.address6.name firewall.addrgrp6.name)"
        ipv6-start-ip: "<your_own_value>"
        keepalive: "74"
        keylife: "75"
        local-gw: "<your_own_value>"
        localid: "<your_own_value>"
        localid-type: "auto"
        mesh-selector-type: "disable"
        mode: "aggressive"
        mode-cfg: "disable"
        name: "default_name_82"
        nattraversal: "enable"
        negotiate-timeout: "84"
        npu-offload: "enable"
        peer: "<your_own_value> (source user.peer.name)"
        peergrp: "<your_own_value> (source user.peergrp.name)"
        peerid: "<your_own_value>"
        peertype: "any"
        ppk: "disable"
        ppk-identity: "<your_own_value>"
        ppk-secret: "<your_own_value>"
        priority: "93"
        proposal: "des-md5"
        psksecret: "<your_own_value>"
        psksecret-remote: "<your_own_value>"
        reauth: "disable"
        rekey: "enable"
        remote-gw: "<your_own_value>"
        remotegw-ddns: "<your_own_value>"
        rsa-signature-format: "pkcs1"
        save-password: "disable"
        send-cert-chain: "enable"
        signature-hash-alg: "sha1"
        split-include-service: "<your_own_value> (source firewall.service.group.name firewall.service.custom.name)"
        suite-b: "disable"
        type: "static"
        unity-support: "disable"
        usrgrp: "<your_own_value> (source user.group.name)"
        wizard-type: "custom"
        xauthtype: "disable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_vpn_ipsec_phase1_data(json):
    option_list = ['acct-verify', 'add-gw-route', 'add-route',
                   'assign-ip', 'assign-ip-from', 'authmethod',
                   'authmethod-remote', 'authpasswd', 'authusr',
                   'authusrgrp', 'auto-negotiate', 'backup-gateway',
                   'banner', 'cert-id-validation', 'certificate',
                   'childless-ike', 'client-auto-negotiate', 'client-keep-alive',
                   'comments', 'dhgrp', 'digital-signature-auth',
                   'distance', 'dns-mode', 'domain',
                   'dpd', 'dpd-retrycount', 'dpd-retryinterval',
                   'eap', 'eap-identity', 'enforce-unique-id',
                   'forticlient-enforcement', 'fragmentation', 'fragmentation-mtu',
                   'group-authentication', 'group-authentication-secret', 'ha-sync-esp-seqno',
                   'idle-timeout', 'idle-timeoutinterval', 'ike-version',
                   'include-local-lan', 'interface', 'ipv4-dns-server1',
                   'ipv4-dns-server2', 'ipv4-dns-server3', 'ipv4-end-ip',
                   'ipv4-exclude-range', 'ipv4-name', 'ipv4-netmask',
                   'ipv4-split-exclude', 'ipv4-split-include', 'ipv4-start-ip',
                   'ipv4-wins-server1', 'ipv4-wins-server2', 'ipv6-dns-server1',
                   'ipv6-dns-server2', 'ipv6-dns-server3', 'ipv6-end-ip',
                   'ipv6-exclude-range', 'ipv6-name', 'ipv6-prefix',
                   'ipv6-split-exclude', 'ipv6-split-include', 'ipv6-start-ip',
                   'keepalive', 'keylife', 'local-gw',
                   'localid', 'localid-type', 'mesh-selector-type',
                   'mode', 'mode-cfg', 'name',
                   'nattraversal', 'negotiate-timeout', 'npu-offload',
                   'peer', 'peergrp', 'peerid',
                   'peertype', 'ppk', 'ppk-identity',
                   'ppk-secret', 'priority', 'proposal',
                   'psksecret', 'psksecret-remote', 'reauth',
                   'rekey', 'remote-gw', 'remotegw-ddns',
                   'rsa-signature-format', 'save-password', 'send-cert-chain',
                   'signature-hash-alg', 'split-include-service', 'suite-b',
                   'type', 'unity-support', 'usrgrp',
                   'wizard-type', 'xauthtype']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def vpn_ipsec_phase1(data, fos):
    vdom = data['vdom']
    vpn_ipsec_phase1_data = data['vpn_ipsec_phase1']
    flattened_data = flatten_multilists_attributes(vpn_ipsec_phase1_data)
    filtered_data = filter_vpn_ipsec_phase1_data(flattened_data)
    if vpn_ipsec_phase1_data['state'] == "present":
        return fos.set('vpn.ipsec',
                       'phase1',
                       data=filtered_data,
                       vdom=vdom)

    elif vpn_ipsec_phase1_data['state'] == "absent":
        return fos.delete('vpn.ipsec',
                          'phase1',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_vpn_ipsec(data, fos):
    login(data)

    if data['vpn_ipsec_phase1']:
        resp = vpn_ipsec_phase1(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "vpn_ipsec_phase1": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "acct-verify": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "add-gw-route": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "add-route": {"required": False, "type": "str",
                              "choices": ["disable", "enable"]},
                "assign-ip": {"required": False, "type": "str",
                              "choices": ["disable", "enable"]},
                "assign-ip-from": {"required": False, "type": "str",
                                   "choices": ["range", "usrgrp", "dhcp",
                                               "name"]},
                "authmethod": {"required": False, "type": "str",
                               "choices": ["psk", "signature"]},
                "authmethod-remote": {"required": False, "type": "str",
                                      "choices": ["psk", "signature"]},
                "authpasswd": {"required": False, "type": "str"},
                "authusr": {"required": False, "type": "str"},
                "authusrgrp": {"required": False, "type": "str"},
                "auto-negotiate": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "backup-gateway": {"required": False, "type": "list",
                                   "options": {
                                       "address": {"required": True, "type": "str"}
                                   }},
                "banner": {"required": False, "type": "str"},
                "cert-id-validation": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "certificate": {"required": False, "type": "list",
                                "options": {
                                    "name": {"required": True, "type": "str"}
                                }},
                "childless-ike": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "client-auto-negotiate": {"required": False, "type": "str",
                                          "choices": ["disable", "enable"]},
                "client-keep-alive": {"required": False, "type": "str",
                                      "choices": ["disable", "enable"]},
                "comments": {"required": False, "type": "str"},
                "dhgrp": {"required": False, "type": "str",
                          "choices": ["1", "2", "5",
                                      "14", "15", "16",
                                      "17", "18", "19",
                                      "20", "21", "27",
                                      "28", "29", "30",
                                      "31"]},
                "digital-signature-auth": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "distance": {"required": False, "type": "int"},
                "dns-mode": {"required": False, "type": "str",
                             "choices": ["manual", "auto"]},
                "domain": {"required": False, "type": "str"},
                "dpd": {"required": False, "type": "str",
                        "choices": ["disable", "on-idle", "on-demand"]},
                "dpd-retrycount": {"required": False, "type": "int"},
                "dpd-retryinterval": {"required": False, "type": "str"},
                "eap": {"required": False, "type": "str",
                        "choices": ["enable", "disable"]},
                "eap-identity": {"required": False, "type": "str",
                                 "choices": ["use-id-payload", "send-request"]},
                "enforce-unique-id": {"required": False, "type": "str",
                                      "choices": ["disable", "keep-new", "keep-old"]},
                "forticlient-enforcement": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "fragmentation": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "fragmentation-mtu": {"required": False, "type": "int"},
                "group-authentication": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "group-authentication-secret": {"required": False, "type": "password-3"},
                "ha-sync-esp-seqno": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "idle-timeout": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "idle-timeoutinterval": {"required": False, "type": "int"},
                "ike-version": {"required": False, "type": "str",
                                "choices": ["1", "2"]},
                "include-local-lan": {"required": False, "type": "str",
                                      "choices": ["disable", "enable"]},
                "interface": {"required": False, "type": "str"},
                "ipv4-dns-server1": {"required": False, "type": "str"},
                "ipv4-dns-server2": {"required": False, "type": "str"},
                "ipv4-dns-server3": {"required": False, "type": "str"},
                "ipv4-end-ip": {"required": False, "type": "str"},
                "ipv4-exclude-range": {"required": False, "type": "list",
                                       "options": {
                                           "end-ip": {"required": False, "type": "str"},
                                           "id": {"required": True, "type": "int"},
                                           "start-ip": {"required": False, "type": "str"}
                                       }},
                "ipv4-name": {"required": False, "type": "str"},
                "ipv4-netmask": {"required": False, "type": "str"},
                "ipv4-split-exclude": {"required": False, "type": "str"},
                "ipv4-split-include": {"required": False, "type": "str"},
                "ipv4-start-ip": {"required": False, "type": "str"},
                "ipv4-wins-server1": {"required": False, "type": "str"},
                "ipv4-wins-server2": {"required": False, "type": "str"},
                "ipv6-dns-server1": {"required": False, "type": "str"},
                "ipv6-dns-server2": {"required": False, "type": "str"},
                "ipv6-dns-server3": {"required": False, "type": "str"},
                "ipv6-end-ip": {"required": False, "type": "str"},
                "ipv6-exclude-range": {"required": False, "type": "list",
                                       "options": {
                                           "end-ip": {"required": False, "type": "str"},
                                           "id": {"required": True, "type": "int"},
                                           "start-ip": {"required": False, "type": "str"}
                                       }},
                "ipv6-name": {"required": False, "type": "str"},
                "ipv6-prefix": {"required": False, "type": "int"},
                "ipv6-split-exclude": {"required": False, "type": "str"},
                "ipv6-split-include": {"required": False, "type": "str"},
                "ipv6-start-ip": {"required": False, "type": "str"},
                "keepalive": {"required": False, "type": "int"},
                "keylife": {"required": False, "type": "int"},
                "local-gw": {"required": False, "type": "str"},
                "localid": {"required": False, "type": "str"},
                "localid-type": {"required": False, "type": "str",
                                 "choices": ["auto", "fqdn", "user-fqdn",
                                             "keyid", "address", "asn1dn"]},
                "mesh-selector-type": {"required": False, "type": "str",
                                       "choices": ["disable", "subnet", "host"]},
                "mode": {"required": False, "type": "str",
                         "choices": ["aggressive", "main"]},
                "mode-cfg": {"required": False, "type": "str",
                             "choices": ["disable", "enable"]},
                "name": {"required": True, "type": "str"},
                "nattraversal": {"required": False, "type": "str",
                                 "choices": ["enable", "disable", "forced"]},
                "negotiate-timeout": {"required": False, "type": "int"},
                "npu-offload": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "peer": {"required": False, "type": "str"},
                "peergrp": {"required": False, "type": "str"},
                "peerid": {"required": False, "type": "str"},
                "peertype": {"required": False, "type": "str",
                             "choices": ["any", "one", "dialup",
                                         "peer", "peergrp"]},
                "ppk": {"required": False, "type": "str",
                        "choices": ["disable", "allow", "require"]},
                "ppk-identity": {"required": False, "type": "str"},
                "ppk-secret": {"required": False, "type": "password-3"},
                "priority": {"required": False, "type": "int"},
                "proposal": {"required": False, "type": "str",
                             "choices": ["des-md5", "des-sha1", "des-sha256",
                                         "des-sha384", "des-sha512"]},
                "psksecret": {"required": False, "type": "password-3"},
                "psksecret-remote": {"required": False, "type": "password-3"},
                "reauth": {"required": False, "type": "str",
                           "choices": ["disable", "enable"]},
                "rekey": {"required": False, "type": "str",
                          "choices": ["enable", "disable"]},
                "remote-gw": {"required": False, "type": "str"},
                "remotegw-ddns": {"required": False, "type": "str"},
                "rsa-signature-format": {"required": False, "type": "str",
                                         "choices": ["pkcs1", "pss"]},
                "save-password": {"required": False, "type": "str",
                                  "choices": ["disable", "enable"]},
                "send-cert-chain": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "signature-hash-alg": {"required": False, "type": "str",
                                       "choices": ["sha1", "sha2-256", "sha2-384",
                                                   "sha2-512"]},
                "split-include-service": {"required": False, "type": "str"},
                "suite-b": {"required": False, "type": "str",
                            "choices": ["disable", "suite-b-gcm-128", "suite-b-gcm-256"]},
                "type": {"required": False, "type": "str",
                         "choices": ["static", "dynamic", "ddns"]},
                "unity-support": {"required": False, "type": "str",
                                  "choices": ["disable", "enable"]},
                "usrgrp": {"required": False, "type": "str"},
                "wizard-type": {"required": False, "type": "str",
                                "choices": ["custom", "dialup-forticlient", "dialup-ios",
                                            "dialup-android", "dialup-windows", "dialup-cisco",
                                            "static-fortigate", "dialup-fortigate", "static-cisco",
                                            "dialup-cisco-fw"]},
                "xauthtype": {"required": False, "type": "str",
                              "choices": ["disable", "client", "pap",
                                          "chap", "auto"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_vpn_ipsec(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2018, Abhijeet Kasurde <akasurde@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}


DOCUMENTATION = '''
---
module: digital_ocean_image_facts
short_description: Gather facts about DigitalOcean images
description:
    - This module can be used to gather facts about DigitalOcean provided images.
    - These images can be either of type C(distribution), C(application) and C(private).
author: "Abhijeet Kasurde (@Akasurde)"
version_added: "2.6"
options:
  image_type:
    description:
     - Specifies the type of image facts to be retrived.
     - If set to C(application), then facts are gathered related to all application images.
     - If set to C(distribution), then facts are gathered related to all distribution images.
     - If set to C(private), then facts are gathered related to all private images.
     - If not set to any of above, then facts are gathered related to all images.
    default: 'all'
    choices: [ 'all', 'application', 'distribution', 'private' ]
    required: false
requirements:
  - "python >= 2.6"
extends_documentation_fragment: digital_ocean.documentation
'''


EXAMPLES = '''
- name: Gather facts about all images
  digital_ocean_image_facts:
    image_type: all
    oauth_token: "{{ oauth_token }}"

- name: Gather facts about application images
  digital_ocean_image_facts:
    image_type: application
    oauth_token: "{{ oauth_token }}"

- name: Gather facts about distribution images
  digital_ocean_image_facts:
    image_type: distribution
    oauth_token: "{{ oauth_token }}"

- name: Get distribution about image with slug coreos-beta
  digital_ocean_image_facts:
  register: resp_out
- set_fact:
    distribution_name: "{{ item.distribution }}"
  loop: "{{ resp_out.data|json_query(name) }}"
  vars:
    name: "[?slug=='coreos-beta']"
- debug: var=distribution_name

'''


RETURN = '''
data:
    description: DigitalOcean image facts
    returned: success
    type: list
    sample: [
        {
            "created_at": "2018-02-02T07:11:43Z",
            "distribution": "CoreOS",
            "id": 31434061,
            "min_disk_size": 20,
            "name": "1662.1.0 (beta)",
            "public": true,
            "regions": [
                "nyc1",
                "sfo1",
                "nyc2",
                "ams2",
                "sgp1",
                "lon1",
                "nyc3",
                "ams3",
                "fra1",
                "tor1",
                "sfo2",
                "blr1"
            ],
            "size_gigabytes": 0.42,
            "slug": "coreos-beta",
            "type": "snapshot"
        },
    ]
'''

from traceback import format_exc
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.digital_ocean import DigitalOceanHelper
from ansible.module_utils._text import to_native


def core(module):
    image_type = module.params['image_type']

    rest = DigitalOceanHelper(module)

    base_url = 'images?'
    if image_type == 'distribution':
        base_url += "type=distribution&"
    elif image_type == 'application':
        base_url += "type=application&"
    elif image_type == 'private':
        base_url += "private=true&"

    images = rest.get_paginated_data(base_url=base_url, data_key_name='images')

    module.exit_json(changed=False, data=images)


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        image_type=dict(type='str',
                        required=False,
                        choices=['all', 'application', 'distribution', 'private'],
                        default='all'
                        )
    )

    module = AnsibleModule(argument_spec=argument_spec)

    try:
        core(module)
    except Exception as e:
        module.fail_json(msg=to_native(e), exception=format_exc())


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright: (c) 2018, Pluribus Networks
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = """
---
module: pn_access_list
author: "Pluribus Networks (@amitsi)"
version_added: "2.8"
short_description: CLI command to create/delete access-list
description:
  - This module can be used to create and delete an access list.
options:
  pn_cliswitch:
    description:
      - Target switch to run the CLI on.
    required: False
    type: str
  state:
    description:
      - State the action to perform. Use 'present' to create access-list and
        'absent' to delete access-list.
    required: True
    choices: [ "present", "absent"]
  pn_name:
    description:
      - Access List Name.
    required: false
    type: str
  pn_scope:
    description:
      - 'scope. Available valid values - local or fabric.'
    required: false
    choices: ['local', 'fabric']
"""

EXAMPLES = """
- name: access list functionality
  pn_access_list:
    pn_cliswitch: "sw01"
    pn_name: "foo"
    pn_scope: "local"
    state: "present"

- name: access list functionality
  pn_access_list:
    pn_cliswitch: "sw01"
    pn_name: "foo"
    pn_scope: "local"
    state: "absent"

- name: access list functionality
  pn_access_list:
    pn_cliswitch: "sw01"
    pn_name: "foo"
    pn_scope: "fabric"
    state: "present"
"""

RETURN = """
command:
  description: the CLI command run on the target node.
  returned: always
  type: str
stdout:
  description: set of responses from the access-list command.
  returned: always
  type: list
stderr:
  description: set of error responses from the access-list command.
  returned: on error
  type: list
changed:
  description: indicates whether the CLI caused changes on the target.
  returned: always
  type: bool
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.netvisor.pn_nvos import pn_cli, run_cli
from ansible.module_utils.network.netvisor.netvisor import run_commands


def check_cli(module, cli):
    """
    This method checks for idempotency using the access-list-show command.
    If a list with given name exists, return True else False.
    :param module: The Ansible module to fetch input parameters
    :param cli: The CLI string
    """
    list_name = module.params['pn_name']

    cli += ' access-list-show format name no-show-headers'
    out = run_commands(module, cli)

    return True if list_name in out else False


def main():
    """ This section is for arguments parsing """

    state_map = dict(
        present='access-list-create',
        absent='access-list-delete',
    )

    module = AnsibleModule(
        argument_spec=dict(
            pn_cliswitch=dict(required=False, type='str'),
            state=dict(required=True, type='str',
                       choices=state_map.keys()),
            pn_name=dict(required=False, type='str'),
            pn_scope=dict(required=False, type='str',
                          choices=['local', 'fabric']),
        ),
        required_if=(
            ["state", "present", ["pn_name", "pn_scope"]],
            ["state", "absent", ["pn_name"]],
        ),
    )

    # Accessing the arguments
    cliswitch = module.params['pn_cliswitch']
    state = module.params['state']
    list_name = module.params['pn_name']
    scope = module.params['pn_scope']

    command = state_map[state]

    # Building the CLI command string
    cli = pn_cli(module, cliswitch)

    ACC_LIST_EXISTS = check_cli(module, cli)
    cli += ' %s name %s ' % (command, list_name)

    if command == 'access-list-delete':
        if ACC_LIST_EXISTS is False:
            module.exit_json(
                skipped=True,
                msg='access-list with name %s does not exist' % list_name
            )
    else:
        if command == 'access-list-create':
            if ACC_LIST_EXISTS is True:
                module.exit_json(
                    skipped=True,
                    msg='access list with name %s already exists' % list_name
                )
        cli += ' scope %s ' % scope

    run_cli(module, cli, state_map)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright (c) 2019 Ansible Project
# Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

PARAMIKO_IMPORT_ERR = None

paramiko = None
try:
    import paramiko
except (ImportError, AttributeError) as err:  # paramiko and gssapi are incompatible and raise AttributeError not ImportError
    PARAMIKO_IMPORT_ERR = err



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_application_list
short_description: Configure application control lists.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure application feature and list category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    application_list:
        description:
            - Configure application control lists.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            app-replacemsg:
                description:
                    - Enable/disable replacement messages for blocked applications.
                choices:
                    - disable
                    - enable
            comment:
                description:
                    - comments
            deep-app-inspection:
                description:
                    - Enable/disable deep application inspection.
                choices:
                    - disable
                    - enable
            entries:
                description:
                    - Application list entries.
                suboptions:
                    action:
                        description:
                            - Pass or block traffic, or reset connection for traffic from this application.
                        choices:
                            - pass
                            - block
                            - reset
                    application:
                        description:
                            - ID of allowed applications.
                        suboptions:
                            id:
                                description:
                                    - Application IDs.
                                required: true
                    behavior:
                        description:
                            - Application behavior filter.
                    category:
                        description:
                            - Category ID list.
                        suboptions:
                            id:
                                description:
                                    - Application category ID.
                                required: true
                    id:
                        description:
                            - Entry ID.
                        required: true
                    log:
                        description:
                            - Enable/disable logging for this application list.
                        choices:
                            - disable
                            - enable
                    log-packet:
                        description:
                            - Enable/disable packet logging.
                        choices:
                            - disable
                            - enable
                    parameters:
                        description:
                            - Application parameters.
                        suboptions:
                            id:
                                description:
                                    - Parameter ID.
                                required: true
                            value:
                                description:
                                    - Parameter value.
                    per-ip-shaper:
                        description:
                            - Per-IP traffic shaper. Source firewall.shaper.per-ip-shaper.name.
                    popularity:
                        description:
                            - Application popularity filter (1 - 5, from least to most popular).
                        choices:
                            - 1
                            - 2
                            - 3
                            - 4
                            - 5
                    protocols:
                        description:
                            - Application protocol filter.
                    quarantine:
                        description:
                            - Quarantine method.
                        choices:
                            - none
                            - attacker
                    quarantine-expiry:
                        description:
                            - Duration of quarantine. (Format ###d##h##m, minimum 1m, maximum 364d23h59m, default = 5m). Requires quarantine set to attacker.
                    quarantine-log:
                        description:
                            - Enable/disable quarantine logging.
                        choices:
                            - disable
                            - enable
                    rate-count:
                        description:
                            - Count of the rate.
                    rate-duration:
                        description:
                            - Duration (sec) of the rate.
                    rate-mode:
                        description:
                            - Rate limit mode.
                        choices:
                            - periodical
                            - continuous
                    rate-track:
                        description:
                            - Track the packet protocol field.
                        choices:
                            - none
                            - src-ip
                            - dest-ip
                            - dhcp-client-mac
                            - dns-domain
                    risk:
                        description:
                            - Risk, or impact, of allowing traffic from this application to occur (1 - 5; Low, Elevated, Medium, High, and Critical).
                        suboptions:
                            level:
                                description:
                                    - Risk, or impact, of allowing traffic from this application to occur (1 - 5; Low, Elevated, Medium, High, and Critical).
                                required: true
                    session-ttl:
                        description:
                            - Session TTL (0 = default).
                    shaper:
                        description:
                            - Traffic shaper. Source firewall.shaper.traffic-shaper.name.
                    shaper-reverse:
                        description:
                            - Reverse traffic shaper. Source firewall.shaper.traffic-shaper.name.
                    sub-category:
                        description:
                            - Application Sub-category ID list.
                        suboptions:
                            id:
                                description:
                                    - Application sub-category ID.
                                required: true
                    technology:
                        description:
                            - Application technology filter.
                    vendor:
                        description:
                            - Application vendor filter.
            extended-log:
                description:
                    - Enable/disable extended logging.
                choices:
                    - enable
                    - disable
            name:
                description:
                    - List name.
                required: true
            options:
                description:
                    - Basic application protocol signatures allowed by default.
                choices:
                    - allow-dns
                    - allow-icmp
                    - allow-http
                    - allow-ssl
                    - allow-quic
            other-application-action:
                description:
                    - Action for other applications.
                choices:
                    - pass
                    - block
            other-application-log:
                description:
                    - Enable/disable logging for other applications.
                choices:
                    - disable
                    - enable
            p2p-black-list:
                description:
                    - P2P applications to be black listed.
                choices:
                    - skype
                    - edonkey
                    - bittorrent
            replacemsg-group:
                description:
                    - Replacement message group. Source system.replacemsg-group.name.
            unknown-application-action:
                description:
                    - Pass or block traffic from unknown applications.
                choices:
                    - pass
                    - block
            unknown-application-log:
                description:
                    - Enable/disable logging for unknown applications.
                choices:
                    - disable
                    - enable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure application control lists.
    fortios_application_list:
      host:  "{{  host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{  vdom }}"
      application_list:
        state: "present"
        app-replacemsg: "disable"
        comment: "comments"
        deep-app-inspection: "disable"
        entries:
         -
            action: "pass"
            application:
             -
                id:  "9"
            behavior: "<your_own_value>"
            category:
             -
                id:  "12"
            id:  "13"
            log: "disable"
            log-packet: "disable"
            parameters:
             -
                id:  "17"
                value: "<your_own_value>"
            per-ip-shaper: "<your_own_value> (source firewall.shaper.per-ip-shaper.name)"
            popularity: "1"
            protocols: "<your_own_value>"
            quarantine: "none"
            quarantine-expiry: "<your_own_value>"
            quarantine-log: "disable"
            rate-count: "25"
            rate-duration: "26"
            rate-mode: "periodical"
            rate-track: "none"
            risk:
             -
                level: "30"
            session-ttl: "31"
            shaper: "<your_own_value> (source firewall.shaper.traffic-shaper.name)"
            shaper-reverse: "<your_own_value> (source firewall.shaper.traffic-shaper.name)"
            sub-category:
             -
                id:  "35"
            technology: "<your_own_value>"
            vendor: "<your_own_value>"
        extended-log: "enable"
        name: "default_name_39"
        options: "allow-dns"
        other-application-action: "pass"
        other-application-log: "disable"
        p2p-black-list: "skype"
        replacemsg-group: "<your_own_value> (source system.replacemsg-group.name)"
        unknown-application-action: "pass"
        unknown-application-log: "disable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_application_list_data(json):
    option_list = ['app-replacemsg', 'comment', 'deep-app-inspection',
                   'entries', 'extended-log', 'name',
                   'options', 'other-application-action', 'other-application-log',
                   'p2p-black-list', 'replacemsg-group', 'unknown-application-action',
                   'unknown-application-log']
    dictionary = {}

    for attribute in option_list:
        if attribute in json:
            dictionary[attribute] = json[attribute]

    return dictionary


def application_list(data, fos):
    vdom = data['vdom']
    application_list_data = data['application_list']
    filtered_data = filter_application_list_data(application_list_data)
    if application_list_data['state'] == "present":
        return fos.set('application',
                       'list',
                       data=filtered_data,
                       vdom=vdom)

    elif application_list_data['state'] == "absent":
        return fos.delete('application',
                          'list',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_application(data, fos):
    login(data)

    methodlist = ['application_list']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "application_list": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "app-replacemsg": {"required": False, "type": "str",
                                   "choices": ["disable", "enable"]},
                "comment": {"required": False, "type": "str"},
                "deep-app-inspection": {"required": False, "type": "str",
                                        "choices": ["disable", "enable"]},
                "entries": {"required": False, "type": "list",
                            "options": {
                                "action": {"required": False, "type": "str",
                                           "choices": ["pass", "block", "reset"]},
                                "application": {"required": False, "type": "list",
                                                "options": {
                                                    "id": {"required": True, "type": "int"}
                                                }},
                                "behavior": {"required": False, "type": "str"},
                                "category": {"required": False, "type": "list",
                                             "options": {
                                                 "id": {"required": True, "type": "int"}
                                             }},
                                "id": {"required": True, "type": "int"},
                                "log": {"required": False, "type": "str",
                                        "choices": ["disable", "enable"]},
                                "log-packet": {"required": False, "type": "str",
                                               "choices": ["disable", "enable"]},
                                "parameters": {"required": False, "type": "list",
                                               "options": {
                                                   "id": {"required": True, "type": "int"},
                                                   "value": {"required": False, "type": "str"}
                                               }},
                                "per-ip-shaper": {"required": False, "type": "str"},
                                "popularity": {"required": False, "type": "str",
                                               "choices": ["1", "2", "3",
                                                           "4", "5"]},
                                "protocols": {"required": False, "type": "str"},
                                "quarantine": {"required": False, "type": "str",
                                               "choices": ["none", "attacker"]},
                                "quarantine-expiry": {"required": False, "type": "str"},
                                "quarantine-log": {"required": False, "type": "str",
                                                   "choices": ["disable", "enable"]},
                                "rate-count": {"required": False, "type": "int"},
                                "rate-duration": {"required": False, "type": "int"},
                                "rate-mode": {"required": False, "type": "str",
                                              "choices": ["periodical", "continuous"]},
                                "rate-track": {"required": False, "type": "str",
                                               "choices": ["none", "src-ip", "dest-ip",
                                                           "dhcp-client-mac", "dns-domain"]},
                                "risk": {"required": False, "type": "list",
                                         "options": {
                                             "level": {"required": True, "type": "int"}
                                         }},
                                "session-ttl": {"required": False, "type": "int"},
                                "shaper": {"required": False, "type": "str"},
                                "shaper-reverse": {"required": False, "type": "str"},
                                "sub-category": {"required": False, "type": "list",
                                                 "options": {
                                                     "id": {"required": True, "type": "int"}
                                                 }},
                                "technology": {"required": False, "type": "str"},
                                "vendor": {"required": False, "type": "str"}
                            }},
                "extended-log": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "name": {"required": True, "type": "str"},
                "options": {"required": False, "type": "str",
                            "choices": ["allow-dns", "allow-icmp", "allow-http",
                                        "allow-ssl", "allow-quic"]},
                "other-application-action": {"required": False, "type": "str",
                                             "choices": ["pass", "block"]},
                "other-application-log": {"required": False, "type": "str",
                                          "choices": ["disable", "enable"]},
                "p2p-black-list": {"required": False, "type": "str",
                                   "choices": ["skype", "edonkey", "bittorrent"]},
                "replacemsg-group": {"required": False, "type": "str"},
                "unknown-application-action": {"required": False, "type": "str",
                                               "choices": ["pass", "block"]},
                "unknown-application-log": {"required": False, "type": "str",
                                            "choices": ["disable", "enable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_application(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_log_disk_setting
short_description: Settings for local disk logging in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify log_disk feature and setting category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    log_disk_setting:
        description:
            - Settings for local disk logging.
        default: null
        suboptions:
            diskfull:
                description:
                    - Action to take when disk is full. The system can overwrite the oldest log messages or stop logging when the disk is full (default =
                       overwrite).
                choices:
                    - overwrite
                    - nolog
            dlp-archive-quota:
                description:
                    - DLP archive quota (MB).
            full-final-warning-threshold:
                description:
                    - Log full final warning threshold as a percent (3 - 100, default = 95).
            full-first-warning-threshold:
                description:
                    - Log full first warning threshold as a percent (1 - 98, default = 75).
            full-second-warning-threshold:
                description:
                    - Log full second warning threshold as a percent (2 - 99, default = 90).
            ips-archive:
                description:
                    - Enable/disable IPS packet archiving to the local disk.
                choices:
                    - enable
                    - disable
            log-quota:
                description:
                    - Disk log quota (MB).
            max-log-file-size:
                description:
                    - Maximum log file size before rolling (1 - 100 Mbytes).
            max-policy-packet-capture-size:
                description:
                    - Maximum size of policy sniffer in MB (0 means unlimited).
            maximum-log-age:
                description:
                    - Delete log files older than (days).
            report-quota:
                description:
                    - Report quota (MB).
            roll-day:
                description:
                    - Day of week on which to roll log file.
                choices:
                    - sunday
                    - monday
                    - tuesday
                    - wednesday
                    - thursday
                    - friday
                    - saturday
            roll-schedule:
                description:
                    - Frequency to check log file for rolling.
                choices:
                    - daily
                    - weekly
            roll-time:
                description:
                    - "Time of day to roll the log file (hh:mm)."
            source-ip:
                description:
                    - Source IP address to use for uploading disk log files.
            status:
                description:
                    - Enable/disable local disk logging.
                choices:
                    - enable
                    - disable
            upload:
                description:
                    - Enable/disable uploading log files when they are rolled.
                choices:
                    - enable
                    - disable
            upload-delete-files:
                description:
                    - Delete log files after uploading (default = enable).
                choices:
                    - enable
                    - disable
            upload-destination:
                description:
                    - The type of server to upload log files to. Only FTP is currently supported.
                choices:
                    - ftp-server
            upload-ssl-conn:
                description:
                    - Enable/disable encrypted FTPS communication to upload log files.
                choices:
                    - default
                    - high
                    - low
                    - disable
            uploaddir:
                description:
                    - The remote directory on the FTP server to upload log files to.
            uploadip:
                description:
                    - IP address of the FTP server to upload log files to.
            uploadpass:
                description:
                    - Password required to log into the FTP server to upload disk log files.
            uploadport:
                description:
                    - TCP port to use for communicating with the FTP server (default = 21).
            uploadsched:
                description:
                    - Set the schedule for uploading log files to the FTP server (default = disable = upload when rolling).
                choices:
                    - disable
                    - enable
            uploadtime:
                description:
                    - "Time of day at which log files are uploaded if uploadsched is enabled (hh:mm or hh)."
            uploadtype:
                description:
                    - Types of log files to upload. Separate multiple entries with a space.
                choices:
                    - traffic
                    - event
                    - virus
                    - webfilter
                    - IPS
                    - spamfilter
                    - dlp-archive
                    - anomaly
                    - voip
                    - dlp
                    - app-ctrl
                    - waf
                    - netscan
                    - gtp
                    - dns
            uploaduser:
                description:
                    - Username required to log into the FTP server to upload disk log files.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Settings for local disk logging.
    fortios_log_disk_setting:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      log_disk_setting:
        diskfull: "overwrite"
        dlp-archive-quota: "4"
        full-final-warning-threshold: "5"
        full-first-warning-threshold: "6"
        full-second-warning-threshold: "7"
        ips-archive: "enable"
        log-quota: "9"
        max-log-file-size: "10"
        max-policy-packet-capture-size: "11"
        maximum-log-age: "12"
        report-quota: "13"
        roll-day: "sunday"
        roll-schedule: "daily"
        roll-time: "<your_own_value>"
        source-ip: "84.230.14.43"
        status: "enable"
        upload: "enable"
        upload-delete-files: "enable"
        upload-destination: "ftp-server"
        upload-ssl-conn: "default"
        uploaddir: "<your_own_value>"
        uploadip: "<your_own_value>"
        uploadpass: "<your_own_value>"
        uploadport: "26"
        uploadsched: "disable"
        uploadtime: "<your_own_value>"
        uploadtype: "traffic"
        uploaduser: "<your_own_value>"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_log_disk_setting_data(json):
    option_list = ['diskfull', 'dlp-archive-quota', 'full-final-warning-threshold',
                   'full-first-warning-threshold', 'full-second-warning-threshold', 'ips-archive',
                   'log-quota', 'max-log-file-size', 'max-policy-packet-capture-size',
                   'maximum-log-age', 'report-quota', 'roll-day',
                   'roll-schedule', 'roll-time', 'source-ip',
                   'status', 'upload', 'upload-delete-files',
                   'upload-destination', 'upload-ssl-conn', 'uploaddir',
                   'uploadip', 'uploadpass', 'uploadport',
                   'uploadsched', 'uploadtime', 'uploadtype',
                   'uploaduser']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def log_disk_setting(data, fos):
    vdom = data['vdom']
    log_disk_setting_data = data['log_disk_setting']
    filtered_data = filter_log_disk_setting_data(log_disk_setting_data)
    return fos.set('log.disk',
                   'setting',
                   data=filtered_data,
                   vdom=vdom)


def fortios_log_disk(data, fos):
    login(data)

    methodlist = ['log_disk_setting']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "log_disk_setting": {
            "required": False, "type": "dict",
            "options": {
                "diskfull": {"required": False, "type": "str",
                             "choices": ["overwrite", "nolog"]},
                "dlp-archive-quota": {"required": False, "type": "int"},
                "full-final-warning-threshold": {"required": False, "type": "int"},
                "full-first-warning-threshold": {"required": False, "type": "int"},
                "full-second-warning-threshold": {"required": False, "type": "int"},
                "ips-archive": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "log-quota": {"required": False, "type": "int"},
                "max-log-file-size": {"required": False, "type": "int"},
                "max-policy-packet-capture-size": {"required": False, "type": "int"},
                "maximum-log-age": {"required": False, "type": "int"},
                "report-quota": {"required": False, "type": "int"},
                "roll-day": {"required": False, "type": "str",
                             "choices": ["sunday", "monday", "tuesday",
                                         "wednesday", "thursday", "friday",
                                         "saturday"]},
                "roll-schedule": {"required": False, "type": "str",
                                  "choices": ["daily", "weekly"]},
                "roll-time": {"required": False, "type": "str"},
                "source-ip": {"required": False, "type": "str"},
                "status": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "upload": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "upload-delete-files": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "upload-destination": {"required": False, "type": "str",
                                       "choices": ["ftp-server"]},
                "upload-ssl-conn": {"required": False, "type": "str",
                                    "choices": ["default", "high", "low",
                                                "disable"]},
                "uploaddir": {"required": False, "type": "str"},
                "uploadip": {"required": False, "type": "str"},
                "uploadpass": {"required": False, "type": "str"},
                "uploadport": {"required": False, "type": "int"},
                "uploadsched": {"required": False, "type": "str",
                                "choices": ["disable", "enable"]},
                "uploadtime": {"required": False, "type": "str"},
                "uploadtype": {"required": False, "type": "str",
                               "choices": ["traffic", "event", "virus",
                                           "webfilter", "IPS", "spamfilter",
                                           "dlp-archive", "anomaly", "voip",
                                           "dlp", "app-ctrl", "waf",
                                           "netscan", "gtp", "dns"]},
                "uploaduser": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_log_disk(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# (c) 2018-2019, NetApp, Inc
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
module: na_ontap_snapshot_policy
short_description: NetApp ONTAP manage Snapshot Policy
extends_documentation_fragment:
    - netapp.na_ontap
version_added: '2.8'
author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>
description:
- Create/Delete ONTAP snapshot policies
options:
  state:
    description:
    - If you want to create or delete a snapshot policy.
    choices: ['present', 'absent']
    default: present
  name:
    description:
      Name of the snapshot policy to be managed.
      The maximum string length is 256 characters.
    required: true
  enabled:
    description:
    - Status of the snapshot policy indicating whether the policy will be enabled or disabled.
    type: bool
  comment:
    description:
      A human readable comment attached with the snapshot.
      The size of the comment can be at most 255 characters.
  count:
    description:
      Retention count for the snapshots created by the schedule.
    type: int
  schedule:
    description:
    - schedule to be added inside the policy.
'''
EXAMPLES = """
    - name: create Snapshot policy
      na_ontap_snapshot_policy:
        state: present
        name: ansible2
        schedule: hourly
        count: 150
        enabled: True
        username: "{{ netapp username }}"
        password: "{{ netapp password }}"
        hostname: "{{ netapp hostname }}"
        https: False

    - name: delete Snapshot policy
      na_ontap_snapshot_policy:
        state: absent
        name: ansible2
        username: "{{ netapp username }}"
        password: "{{ netapp password }}"
        hostname: "{{ netapp hostname }}"
        https: False
"""

RETURN = """
"""
import traceback

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.netapp_module import NetAppModule
from ansible.module_utils._text import to_native
import ansible.module_utils.netapp as netapp_utils

HAS_NETAPP_LIB = netapp_utils.has_netapp_lib()


class NetAppOntapSnapshotPolicy(object):
    """
    Creates and deletes a Snapshot Policy
    """

    def __init__(self):
        self.argument_spec = netapp_utils.na_ontap_host_argument_spec()
        self.argument_spec.update(dict(
            state=dict(required=False, choices=[
                       'present', 'absent'], default='present'),
            name=dict(required=True, type="str"),
            enabled=dict(required=False, type="bool"),
            count=dict(required=False, type="int"),
            comment=dict(required=False, type="str"),
            schedule=dict(required=False, type="str")
        ))
        self.module = AnsibleModule(
            argument_spec=self.argument_spec,
            required_if=[
                ('state', 'present', ['enabled', 'count', 'schedule']),
            ],
            supports_check_mode=True
        )

        self.na_helper = NetAppModule()
        self.parameters = self.na_helper.set_parameters(self.module.params)

        if HAS_NETAPP_LIB is False:
            self.module.fail_json(
                msg="the python NetApp-Lib module is required")
        else:
            self.server = netapp_utils.setup_na_ontap_zapi(module=self.module)
        return

    def get_snapshot_policy(self):
        """
        Checks to see if a snapshot policy exists or not
        :return: Return policy details if a snapshot policy exists, None if it doesn't
        """
        snapshot_obj = netapp_utils.zapi.NaElement("snapshot-policy-get-iter")
        # compose query
        query = netapp_utils.zapi.NaElement("query")
        snapshot_info_obj = netapp_utils.zapi.NaElement("snapshot-policy-info")
        snapshot_info_obj.add_new_child("policy", self.parameters['name'])
        query.add_child_elem(snapshot_info_obj)
        snapshot_obj.add_child_elem(query)
        try:
            result = self.server.invoke_successfully(snapshot_obj, True)
            if result.get_child_by_name('num-records') and \
                    int(result.get_child_content('num-records')) == 1:
                return result
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg=to_native(error), exception=traceback.format_exc())
        return None

    def create_snapshot_policy(self):
        """
        Creates a new snapshot policy
        """
        # set up required variables to create a snapshot policy
        options = {'policy': self.parameters['name'],
                   'enabled': str(self.parameters['enabled']),
                   'count1': str(self.parameters['count']),
                   'schedule1': self.parameters['schedule']
                   }
        snapshot_obj = netapp_utils.zapi.NaElement.create_node_with_children('snapshot-policy-create', **options)

        # Set up optional variables to create a snapshot policy
        if self.parameters.get('comment'):
            snapshot_obj.add_new_child("comment", self.parameters['comment'])
        try:
            self.server.invoke_successfully(snapshot_obj, True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error creating snapshot policy %s: %s' %
                                  (self.parameters['name'], to_native(error)),
                                  exception=traceback.format_exc())

    def delete_snapshot_policy(self):
        """
        Deletes an existing snapshot policy
        """
        snapshot_obj = netapp_utils.zapi.NaElement("snapshot-policy-delete")

        # Set up required variables to delete a snapshot policy
        snapshot_obj.add_new_child("policy", self.parameters['name'])
        try:
            self.server.invoke_successfully(snapshot_obj, True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error deleting snapshot policy %s: %s' %
                                  (self.parameters['name'], to_native(error)),
                                  exception=traceback.format_exc())

    def asup_log_for_cserver(self, event_name):
        """
        Fetch admin vserver for the given cluster
        Create and Autosupport log event with the given module name
        :param event_name: Name of the event log
        :return: None
        """
        results = netapp_utils.get_cserver(self.server)
        cserver = netapp_utils.setup_na_ontap_zapi(module=self.module, vserver=results)
        netapp_utils.ems_log_event(event_name, cserver)

    def apply(self):
        """
        Check to see which play we should run
        """
        self.asup_log_for_cserver("na_ontap_snapshot_policy")
        current = self.get_snapshot_policy()
        cd_action = self.na_helper.get_cd_action(current, self.parameters)
        if self.na_helper.changed:
            if self.module.check_mode:
                pass
            else:
                if cd_action == 'create':
                    self.create_snapshot_policy()
                elif cd_action == 'delete':
                    self.delete_snapshot_policy()
        self.module.exit_json(changed=self.na_helper.changed)


def main():
    """
    Creates and deletes a Snapshot Policy
    """
    obj = NetAppOntapSnapshotPolicy()
    obj.apply()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright: (c) 2018, Pluribus Networks
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = """
---
module: pn_admin_syslog
author: "Pluribus Networks (@rajaspachipulusu17)"
version_added: "2.8"
short_description: CLI command to create/modify/delete admin-syslog
description:
  - This module can be used to create the scope and other parameters of syslog event collection.
  - This module can be used to modify parameters of syslog event collection.
  - This module can be used to delete the scope and other parameters of syslog event collection.
options:
  pn_cliswitch:
    description:
      - Target switch to run the CLI on.
    required: False
    type: str
  state:
    description:
      - State the action to perform. Use C(present) to create admin-syslog and
        C(absent) to delete admin-syslog C(update) to modify the admin-syslog.
    required: True
    type: str
    choices: ['present', 'absent', 'update']
  pn_scope:
    description:
      - Scope of the system log.
    required: False
    type: str
    choices: ['local', 'fabric']
  pn_host:
    description:
      - Hostname to log system events.
    required: False
    type: str
  pn_port:
    description:
      - Host port.
    required: False
    type: str
  pn_transport:
    description:
      - Transport for log events - tcp/tls or udp.
    required: False
    type: str
    choices: ['tcp-tls', 'udp']
    default: 'udp'
  pn_message_format:
    description:
      - message-format for log events - structured or legacy.
    required: False
    choices: ['structured', 'legacy']
    type: str
  pn_name:
    description:
      - name of the system log.
    required: False
    type: str
"""

EXAMPLES = """
- name: admin-syslog functionality
  pn_admin_syslog:
    pn_cliswitch: "sw01"
    state: "absent"
    pn_name: "foo"
    pn_scope: "local"

- name: admin-syslog functionality
  pn_admin_syslog:
    pn_cliswitch: "sw01"
    state: "present"
    pn_name: "foo"
    pn_scope: "local"
    pn_host: "166.68.224.46"
    pn_message_format: "structured"

- name: admin-syslog functionality
  pn_admin_syslog:
    pn_cliswitch: "sw01"
    state: "update"
    pn_name: "foo"
    pn_host: "166.68.224.10"
"""

RETURN = """
command:
  description: the CLI command run on the target node.
  returned: always
  type: str
stdout:
  description: set of responses from the admin-syslog command.
  returned: always
  type: list
stderr:
  description: set of error responses from the admin-syslog command.
  returned: on error
  type: list
changed:
  description: indicates whether the CLI caused changes on the target.
  returned: always
  type: bool
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.netvisor.pn_nvos import pn_cli, run_cli
from ansible.module_utils.network.netvisor.netvisor import run_commands


def check_cli(module, cli):
    """
    This method checks for idempotency using the admin-syslog-show command.
    If a user with given name exists, return as True else False.
    :param module: The Ansible module to fetch input parameters
    :param cli: The CLI string
    """

    name = module.params['pn_name']

    cli += ' admin-syslog-show format name no-show-headers'
    out = run_commands(module, cli)[1]

    out = out.split()

    return True if name in out else False


def main():
    """ This section is for arguments parsing """

    state_map = dict(
        present='admin-syslog-create',
        absent='admin-syslog-delete',
        update='admin-syslog-modify'
    )

    module = AnsibleModule(
        argument_spec=dict(
            pn_cliswitch=dict(required=False, type='str'),
            state=dict(required=True, type='str',
                       choices=state_map.keys()),
            pn_scope=dict(required=False, type='str',
                          choices=['local', 'fabric']),
            pn_host=dict(required=False, type='str'),
            pn_port=dict(required=False, type='str'),
            pn_transport=dict(required=False, type='str',
                              choices=['tcp-tls', 'udp'], default='udp'),
            pn_message_format=dict(required=False, type='str',
                                   choices=['structured', 'legacy']),
            pn_name=dict(required=False, type='str'),
        ),
        required_if=(
            ['state', 'present', ['pn_name', 'pn_host', 'pn_scope']],
            ['state', 'absent', ['pn_name']],
            ['state', 'update', ['pn_name']]
        ),
        required_one_of=[['pn_port', 'pn_message_format',
                          'pn_host', 'pn_transport', 'pn_scope']]
    )

    # Accessing the arguments
    cliswitch = module.params['pn_cliswitch']
    state = module.params['state']
    scope = module.params['pn_scope']
    host = module.params['pn_host']
    port = module.params['pn_port']
    transport = module.params['pn_transport']
    message_format = module.params['pn_message_format']
    name = module.params['pn_name']

    command = state_map[state]

    # Building the CLI command string
    cli = pn_cli(module, cliswitch)

    SYSLOG_EXISTS = check_cli(module, cli)
    cli += ' %s name %s ' % (command, name)

    if command == 'admin-syslog-modify':
        if SYSLOG_EXISTS is False:
            module.fail_json(
                failed=True,
                msg='admin syslog with name %s does not exist' % name
            )

    if command == 'admin-syslog-delete':
        if SYSLOG_EXISTS is False:
            module.exit_json(
                skipped=True,
                msg='admin syslog with name %s does not exist' % name
            )

    if command == 'admin-syslog-create':
        if SYSLOG_EXISTS is True:
            module.exit_json(
                skipped=True,
                msg='admin syslog user with name %s already exists' % name
            )

    if command == 'admin-syslog-create':
        if scope:
            cli += ' scope ' + scope

    if command != 'admin-syslog-delete':
        if host:
            cli += ' host ' + host
        if port:
            cli += ' port ' + port
        if transport:
            cli += ' transport ' + transport
        if message_format:
            cli += ' message-format ' + message_format

    run_cli(module, cli, state_map)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2018, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: bigip_asm_policy_import
short_description: Manage BIG-IP ASM policy imports
description:
   - Manage BIG-IP ASM policies policy imports.
version_added: 2.8
options:
  name:
    description:
      - The ASM policy to create or override.
    type: str
    required: True
  inline:
    description:
      - When specified the ASM policy is created from a provided string.
      - Content needs to be provided in a valid XML format otherwise the operation will fail.
    type: str
  source:
    description:
      - Full path to a policy file to be imported into the BIG-IP ASM.
      - Policy files exported from newer versions of BIG-IP cannot be imported into older
        versions of BIG-IP. The opposite, however, is true; you can import older into
        newer.
      - The file format can be binary of XML.
    type: path
  force:
    description:
      - When set to C(yes) any existing policy with the same name will be overwritten by the new import.
      - Works for both inline and file imports, if the policy does not exist this setting is ignored.
    default: no
    type: bool
  partition:
    description:
      - Device partition to create policy on.
    type: str
    default: Common
extends_documentation_fragment: f5
author:
  - Wojciech Wypior (@wojtek0806)
'''

EXAMPLES = r'''
- name: Import ASM policy
  bigip_asm_policy_import:
    name: new_asm_policy
    file: /root/asm_policy.xml
    provider:
      server: lb.mydomain.com
      user: admin
      password: secret
  delegate_to: localhost

- name: Import ASM policy inline
  bigip_asm_policy_import:
    name: foo-policy4
    inline: <xml>content</xml>
    provider:
      server: lb.mydomain.com
      user: admin
      password: secret
  delegate_to: localhost

- name: Override existing ASM policy
  bigip_asm_policy:
    name: new_asm_policy
    file: /root/asm_policy_new.xml
    force: yes
    provider:
      server: lb.mydomain.com
      user: admin
      password: secret
  delegate_to: localhost
'''

RETURN = r'''
source:
  description: Local path to an ASM policy file.
  returned: changed
  type: str
  sample: /root/some_policy.xml
inline:
  description: Contents of policy as an inline string
  returned: changed
  type: str
  sample: <xml>foobar contents</xml>
name:
  description: Name of the ASM policy to be created/overwritten
  returned: changed
  type: str
  sample: Asm_APP1_Transparent
force:
  description: Set when overwriting an existing policy
  returned: changed
  type: bool
  sample: yes
'''

import os
import time
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.basic import env_fallback

try:
    from library.module_utils.network.f5.bigip import F5RestClient
    from library.module_utils.network.f5.common import F5ModuleError
    from library.module_utils.network.f5.common import AnsibleF5Parameters
    from library.module_utils.network.f5.common import fq_name
    from library.module_utils.network.f5.common import transform_name
    from library.module_utils.network.f5.common import f5_argument_spec
    from library.module_utils.network.f5.icontrol import upload_file
    from library.module_utils.network.f5.icontrol import module_provisioned
except ImportError:
    from ansible.module_utils.network.f5.bigip import F5RestClient
    from ansible.module_utils.network.f5.common import F5ModuleError
    from ansible.module_utils.network.f5.common import AnsibleF5Parameters
    from ansible.module_utils.network.f5.common import fq_name
    from ansible.module_utils.network.f5.common import transform_name
    from ansible.module_utils.network.f5.common import f5_argument_spec
    from ansible.module_utils.network.f5.icontrol import upload_file
    from ansible.module_utils.network.f5.icontrol import module_provisioned


class Parameters(AnsibleF5Parameters):
    updatables = []

    returnables = [
        'name',
        'inline',
        'source',
        'force'
    ]

    api_attributes = [
        'file',
        'name',
    ]

    api_map = {
        'file': 'inline',
        'filename': 'source',
    }


class ApiParameters(Parameters):
    pass


class ModuleParameters(Parameters):
    pass


class Changes(Parameters):
    def to_return(self):
        result = {}
        try:
            for returnable in self.returnables:
                result[returnable] = getattr(self, returnable)
            result = self._filter_params(result)
        except Exception:
            pass
        return result


class UsableChanges(Changes):
    pass


class ReportableChanges(Changes):
    pass


class Difference(object):
    def __init__(self, want, have=None):
        self.want = want
        self.have = have

    def compare(self, param):
        try:
            result = getattr(self, param)
            return result
        except AttributeError:
            return self.__default(param)

    def __default(self, param):
        attr1 = getattr(self.want, param)
        try:
            attr2 = getattr(self.have, param)
            if attr1 != attr2:
                return attr1
        except AttributeError:
            return attr1


class ModuleManager(object):
    def __init__(self, *args, **kwargs):
        self.module = kwargs.get('module', None)
        self.client = F5RestClient(**self.module.params)
        self.want = ModuleParameters(params=self.module.params)
        self.changes = UsableChanges()

    def _set_changed_options(self):
        changed = {}
        for key in Parameters.returnables:
            if getattr(self.want, key) is not None:
                changed[key] = getattr(self.want, key)
        if changed:
            self.changes = UsableChanges(params=changed)

    def _announce_deprecations(self, result):
        warnings = result.pop('__warnings', [])
        for warning in warnings:
            self.client.module.deprecate(
                msg=warning['msg'],
                version=warning['version']
            )

    def exec_module(self):
        if not module_provisioned(self.client, 'asm'):
            raise F5ModuleError(
                "ASM must be provisioned to use this module."
            )

        result = dict()

        changed = self.policy_import()

        reportable = ReportableChanges(params=self.changes.to_return())
        changes = reportable.to_return()
        result.update(**changes)
        result.update(dict(changed=changed))
        self._announce_deprecations(result)
        return result

    def policy_import(self):
        self._set_changed_options()
        if self.module.check_mode:
            return True
        if self.exists():
            if self.want.force is False:
                return False
        if self.want.inline:
            task = self.inline_import()
            self.wait_for_task(task)
            return True

        self.import_file_to_device()
        self.remove_temp_policy_from_device()
        return True

    def exists(self):
        uri = 'https://{0}:{1}/mgmt/tm/asm/policies/'.format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )

        query = "?$filter=contains(name,'{0}')+and+contains(partition,'{1}')&$select=name,partition".format(
            self.want.name, self.want.partition
        )
        resp = self.client.api.get(uri + query)

        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))
        if 'items' in response and response['items'] != []:
            return True
        return False

    def upload_file_to_device(self, content, name):
        url = 'https://{0}:{1}/mgmt/shared/file-transfer/uploads'.format(
            self.client.provider['server'],
            self.client.provider['server_port']
        )
        try:
            upload_file(self.client, url, content, name)
        except F5ModuleError:
            raise F5ModuleError(
                "Failed to upload the file."
            )

    def _get_policy_link(self):
        uri = 'https://{0}:{1}/mgmt/tm/asm/policies/'.format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )

        query = "?$filter=contains(name,'{0}')+and+contains(partition,'{1}')&$select=name,partition".format(
            self.want.name, self.want.partition
        )
        resp = self.client.api.get(uri + query)

        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        policy_link = response['items'][0]['selfLink']
        return policy_link

    def inline_import(self):
        params = self.changes.api_params()
        uri = "https://{0}:{1}/mgmt/tm/asm/tasks/import-policy/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )

        if self.want.force:
            params.update(dict(policyReference={'link': self._get_policy_link()}))
            params.pop('name')

        resp = self.client.api.post(uri, json=params)

        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] in [400, 403]:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return response['id']

    def wait_for_task(self, task_id):
        uri = "https://{0}:{1}/mgmt/tm/asm/tasks/import-policy/{2}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            task_id
        )
        while True:
            resp = self.client.api.get(uri)

            try:
                response = resp.json()
            except ValueError as ex:
                raise F5ModuleError(str(ex))

            if 'code' in response and response['code'] == 400:
                if 'message' in response:
                    raise F5ModuleError(response['message'])
                else:
                    raise F5ModuleError(resp.content)

            if response['status'] in ['COMPLETED', 'FAILURE']:
                break
            time.sleep(1)

        if response['status'] == 'FAILURE':
            raise F5ModuleError(
                'Failed to import ASM policy.'
            )
        if response['status'] == 'COMPLETED':
            return True

    def import_file_to_device(self):
        name = os.path.split(self.want.source)[1]
        self.upload_file_to_device(self.want.source, name)
        time.sleep(2)

        full_name = fq_name(self.want.partition, self.want.name)

        if self.want.force:
            cmd = 'tmsh load asm policy {0} file /var/config/rest/downloads/{1} overwrite'.format(full_name, name)
        else:
            cmd = 'tmsh load asm policy {0} file /var/config/rest/downloads/{1}'.format(full_name, name)

        uri = "https://{0}:{1}/mgmt/tm/util/bash/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        args = dict(
            command='run',
            utilCmdArgs='-c "{0}"'.format(cmd)
        )
        resp = self.client.api.post(uri, json=args)

        try:
            response = resp.json()
            if 'commandResult' in response:
                if 'Unexpected Error' in response['commandResult']:
                    raise F5ModuleError(response['commandResult'])
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return True

    def remove_temp_policy_from_device(self):
        name = os.path.split(self.want.source)[1]
        tpath_name = '/var/config/rest/downloads/{0}'.format(name)
        uri = "https://{0}:{1}/mgmt/tm/util/unix-rm/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        args = dict(
            command='run',
            utilCmdArgs=tpath_name
        )
        resp = self.client.api.post(uri, json=args)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))
        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)


class ArgumentSpec(object):
    def __init__(self):
        self.supports_check_mode = True
        argument_spec = dict(
            name=dict(
                required=True,
            ),
            source=dict(type='path'),
            inline=dict(),
            force=dict(
                type='bool',
                default='no'
            ),
            partition=dict(
                default='Common',
                fallback=(env_fallback, ['F5_PARTITION'])
            )
        )
        self.argument_spec = {}
        self.argument_spec.update(f5_argument_spec)
        self.argument_spec.update(argument_spec)
        self.mutually_exclusive = [
            ['source', 'inline']
        ]


def main():
    spec = ArgumentSpec()

    module = AnsibleModule(
        argument_spec=spec.argument_spec,
        supports_check_mode=spec.supports_check_mode,
        mutually_exclusive=spec.mutually_exclusive
    )

    try:
        mm = ModuleManager(module=module)
        results = mm.exec_module()
        module.exit_json(**results)
    except F5ModuleError as ex:
        module.fail_json(msg=str(ex))


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright: (c) 2018, Rhys Campbell <rhys.james.campbell@googlemail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: mongodb_replicaset
short_description: Initialises a MongoDB replicaset.
description:
- Initialises a MongoDB replicaset in a new deployment.
- Validates the replicaset name for existing deployments.
author: Rhys Campbell (@rhysmeister)
version_added: "2.8"
options:
  login_user:
    description:
    - The username to authenticate with.
    type: str
  login_password:
    description:
    - The password to authenticate with.
    type: str
  login_database:
    description:
    - The database where login credentials are stored.
    type: str
    default: admin
  login_host:
    description:
    - The MongoDB hostname.
    type: str
    default: localhost
  login_port:
    description:
    - The MongoDB port to login to.
    type: int
    default: 27017
  replica_set:
    description:
    - Replicaset name.
    type: str
    default: rs0
  members:
    description:
    - A comma-separated string or a yaml list consisting of the replicaset members.
    - Supply as a simple csv string, i.e. mongodb1:27017,mongodb2:27017,mongodb3:27017.
    - If a port number is not provided then 27017 is assumed.
    type: list
  validate:
    description:
    - Performs some basic validation on the provided replicaset config.
    type: bool
    default: yes
  ssl:
    description:
    - Whether to use an SSL connection when connecting to the database
    type: bool
    default: no
  ssl_cert_reqs:
    description:
    - Specifies whether a certificate is required from the other side of the connection, and whether it will be validated if provided.
    type: str
    default: CERT_REQUIRED
    choices: [ CERT_NONE, CERT_OPTIONAL, CERT_REQUIRED ]
  arbiter_at_index:
    description:
    - Identifies the position of the member in the array that is an arbiter.
    type: int
  chaining_allowed:
    description:
    - When I(settings.chaining_allowed=true), the replicaset allows secondary members to replicate from other
      secondary members.
    - When I(settings.chaining_allowed=false), secondaries can replicate only from the primary.
    type: bool
    default: yes
  heartbeat_timeout_secs:
    description:
    - Number of seconds that the replicaset members wait for a successful heartbeat from each other.
    - If a member does not respond in time, other members mark the delinquent member as inaccessible.
    - The setting only applies when using I(protocol_version=0). When using I(protocol_version=1) the relevant
      setting is I(settings.election_timeout_millis).
    type: int
    default: 10
  election_timeout_millis:
    description:
    - The time limit in milliseconds for detecting when a replicaset's primary is unreachable.
    type: int
    default: 10000
  protocol_version:
    description: Version of the replicaset election protocol.
    type: int
    choices: [ 0, 1 ]
    default: 1
notes:
- Requires the pymongo Python package on the remote host, version 2.4.2+. This
  can be installed using pip or the OS package manager. @see U(http://api.mongodb.org/python/current/installation.html)
requirements:
- pymongo
'''

EXAMPLES = r'''
# Create a replicaset called 'rs0' with the 3 provided members
- name: Ensure replicaset rs0 exists
  mongodb_replicaset:
    login_host: localhost
    login_user: admin
    login_password: admin
    replica_set: rs0
    members:
    - mongodb1:27017
    - mongodb2:27017
    - mongodb3:27017
  when: groups.mongod.index(inventory_hostname) == 0

# Create two single-node replicasets on the localhost for testing
- name: Ensure replicaset rs0 exists
  mongodb_replicaset:
    login_host: localhost
    login_port: 3001
    login_user: admin
    login_password: secret
    login_database: admin
    replica_set: rs0
    members: localhost:3001
    validate: no

- name: Ensure replicaset rs1 exists
  mongodb_replicaset:
    login_host: localhost
    login_port: 3002
    login_user: admin
    login_password: secret
    login_database: admin
    replica_set: rs1
    members: localhost:3002
    validate: no
'''

RETURN = r'''
mongodb_replicaset:
  description: The name of the replicaset that has been created.
  returned: success
  type: str
'''

from copy import deepcopy

import os
import ssl as ssl_lib
from distutils.version import LooseVersion

try:
    from pymongo.errors import ConnectionFailure
    from pymongo.errors import OperationFailure
    from pymongo import version as PyMongoVersion
    from pymongo import MongoClient
    HAS_PYMONGO = True
except ImportError:
    try:  # for older PyMongo 2.2
        from pymongo import Connection as MongoClient
        HAS_PYMONGO = True
    except ImportError:
        HAS_PYMONGO = False

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six import binary_type, text_type
from ansible.module_utils.six.moves import configparser
from ansible.module_utils._text import to_native


# =========================================
# MongoDB module specific support methods.
#

def check_compatibility(module, client):
    """Check the compatibility between the driver and the database.

       See: https://docs.mongodb.com/ecosystem/drivers/driver-compatibility-reference/#python-driver-compatibility

    Args:
        module: Ansible module.
        client (cursor): Mongodb cursor on admin database.
    """
    loose_srv_version = LooseVersion(client.server_info()['version'])
    loose_driver_version = LooseVersion(PyMongoVersion)

    if loose_srv_version >= LooseVersion('3.2') and loose_driver_version < LooseVersion('3.2'):
        module.fail_json(msg=' (Note: you must use pymongo 3.2+ with MongoDB >= 3.2)')

    elif loose_srv_version >= LooseVersion('3.0') and loose_driver_version <= LooseVersion('2.8'):
        module.fail_json(msg=' (Note: you must use pymongo 2.8+ with MongoDB 3.0)')

    elif loose_srv_version >= LooseVersion('2.6') and loose_driver_version <= LooseVersion('2.7'):
        module.fail_json(msg=' (Note: you must use pymongo 2.7+ with MongoDB 2.6)')

    elif LooseVersion(PyMongoVersion) <= LooseVersion('2.5'):
        module.fail_json(msg=' (Note: you must be on mongodb 2.4+ and pymongo 2.5+ to use the roles param)')


def replicaset_find(client):
    """Check if a replicaset exists.

    Args:
        client (cursor): Mongodb cursor on admin database.
        replica_set (str): replica_set to check.

    Returns:
        dict: when user exists, False otherwise.
    """
    for rs in client["local"].system.replset.find({}):
        return rs["_id"]
    return False


def replicaset_add(module, client, replica_set, members, arbiter_at_index, protocol_version,
                   chaining_allowed, heartbeat_timeout_secs, election_timeout_millis):

    try:
        from collections import OrderedDict
    except ImportError as excep:
        try:
            from ordereddict import OrderedDict
        except ImportError as excep:
            module.fail_json(msg='Cannot import OrderedDict class. You can probably install with: pip install ordereddict: %s'
                             % to_native(excep))

    members_dict_list = []
    index = 0
    settings = {
        "chainingAllowed": bool(chaining_allowed),
    }
    if protocol_version == 0:
        settings['heartbeatTimeoutSecs'] = heartbeat_timeout_secs
    else:
        settings['electionTimeoutMillis'] = election_timeout_millis
    for member in members:
        if ':' not in member:  # No port supplied. Assume 27017
            member += ":27017"
        members_dict_list.append(OrderedDict([("_id", index), ("host", member)]))
        if index == arbiter_at_index:
            members_dict_list[index]['arbiterOnly'] = True
        index += 1

    conf = OrderedDict([("_id", replica_set),
                        ("protocolVersion", protocol_version),
                        ("members", members_dict_list),
                        ("settings", settings)])
    client["admin"].command('replSetInitiate', conf)


def replicaset_remove(module, client, replica_set):
    raise NotImplementedError
    # exists = replicaset_find(client, replica_set)
    # if exists:
    #    if module.check_mode:
    #        module.exit_json(changed=True, replica_set=replica_set)
    #    db = client[db_name]
    #    db.remove_user(replica_set)
    # else:
    #    module.exit_json(changed=False, user=user)


def load_mongocnf():
    config = configparser.RawConfigParser()
    mongocnf = os.path.expanduser('~/.mongodb.cnf')

    try:
        config.readfp(open(mongocnf))
    except (configparser.NoOptionError, IOError):
        return False

    creds = dict(
        user=config.get('client', 'user'),
        password=config.get('client', 'pass')
    )

    return creds


# =========================================
# Module execution.
#


def main():
    module = AnsibleModule(
        argument_spec=dict(
            login_user=dict(type='str'),
            login_password=dict(type='str', no_log=True),
            login_database=dict(type='str', default="admin"),
            login_host=dict(type='str', default="localhost"),
            login_port=dict(type='int', default=27017),
            replica_set=dict(type='str', default="rs0"),
            members=dict(type='list'),
            arbiter_at_index=dict(type='int'),
            validate=dict(type='bool', default=True),
            ssl=dict(type='bool', default=False),
            ssl_cert_reqs=dict(type='str', default='CERT_REQUIRED', choices=['CERT_NONE', 'CERT_OPTIONAL', 'CERT_REQUIRED']),
            protocol_version=dict(type='int', default=1, choices=[0, 1]),
            chaining_allowed=dict(type='bool', default=True),
            heartbeat_timeout_secs=dict(type='int', default=10),
            election_timeout_millis=dict(type='int', default=10000)
        ),
        supports_check_mode=True,
    )

    if not HAS_PYMONGO:
        module.fail_json(msg='the python pymongo module is required')

    login_user = module.params['login_user']
    login_password = module.params['login_password']
    login_database = module.params['login_database']
    login_host = module.params['login_host']
    login_port = module.params['login_port']
    replica_set = module.params['replica_set']
    members = module.params['members']
    arbiter_at_index = module.params['arbiter_at_index']
    validate = module.params['validate']
    ssl = module.params['ssl']
    protocol_version = module.params['protocol_version']
    chaining_allowed = module.params['chaining_allowed']
    heartbeat_timeout_secs = module.params['heartbeat_timeout_secs']
    election_timeout_millis = module.params['election_timeout_millis']

    if validate:
        if len(members) <= 2 or len(members) % 2 == 0:
            module.fail_json(msg="MongoDB Replicaset validation failed. Invalid number of replicaset members.")
        if arbiter_at_index is not None and len(members) - 1 > arbiter_at_index:
            module.fail_json(msg="MongoDB Replicaset validation failed. Invalid arbiter index.")

    result = dict(
        changed=False,
        replica_set=replica_set,
    )

    connection_params = dict(
        host=login_host,
        port=int(login_port),
    )

    if ssl:
        connection_params["ssl"] = ssl
        connection_params["ssl_cert_reqs"] = getattr(ssl_lib, module.params['ssl_cert_reqs'])

    try:
        client = MongoClient(**connection_params)
    except Exception as e:
        module.fail_json(msg='Unable to connect to database: %s' % to_native(e))

    try:
        check_compatibility(module, client)
    except Exception as excep:
        if "not authorized on" not in str(excep) and "there are no users authenticated" not in str(excep):
            raise excep
        if login_user is None or login_password is None:
            raise excep
        client.admin.authenticate(login_user, login_password, source=login_database)
        check_compatibility(module, client)

    if login_user is None and login_password is None:
        mongocnf_creds = load_mongocnf()
        if mongocnf_creds is not False:
            login_user = mongocnf_creds['user']
            login_password = mongocnf_creds['password']
    elif login_password is None or login_user is None:
        module.fail_json(msg="When supplying login arguments, both 'login_user' and 'login_password' must be provided")

    try:
        client['admin'].command('listDatabases', 1.0)  # if this throws an error we need to authenticate
    except Exception as excep:
        if "not authorized on" in str(excep) or "command listDatabases requires authentication" in str(excep):
            if login_user is not None and login_password is not None:
                client.admin.authenticate(login_user, login_password, source=login_database)
            else:
                raise excep
        else:
            raise excep

    if len(replica_set) == 0:
        module.fail_json(msg="Parameter 'replica_set' must not be an empty string")

    try:
        rs = replicaset_find(client)
    except Exception as e:
        module.fail_json(msg='Unable to query replica_set info: %s' % to_native(e))

    if not rs:
        if not module.check_mode:
            try:
                replicaset_add(module, client, replica_set, members, arbiter_at_index, protocol_version,
                               chaining_allowed, heartbeat_timeout_secs, election_timeout_millis)
                result['changed'] = True
            except Exception as e:
                module.fail_json(msg='Unable to create replica_set: %s' % to_native(e))
    else:
        if not module.check_mode:
            try:
                rs = replicaset_find(client)
            except Exception as e:
                module.fail_json(msg='Unable to query replica_set info: %s' % to_native(e))
            if rs is not None and rs != replica_set:
                module.fail_json(msg="The replica_set name of '{0}' does not match the expected: '{1}'".format(rs, replica_set))
        result['changed'] = False

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2018, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: bigip_imish_config
short_description: Manage BIG-IP advanced routing configuration sections
description:
  - This module provides an implementation for working with advanced routing
    configuration sections in a deterministic way.
version_added: 2.8
options:
  route_domain:
    description:
      - Route domain to manage BGP configuration on.
    type: str
    default: 0
  lines:
    description:
      - The ordered set of commands that should be configured in the
        section.
      - The commands must be the exact same commands as found in the device
        running-config.
      - Be sure to note the configuration command syntax as some commands
        are automatically modified by the device config parser.
    type: list
    aliases: ['commands']
  parents:
    description:
      - The ordered set of parents that uniquely identify the section or hierarchy
        the commands should be checked against.
      - If the C(parents) argument is omitted, the commands are checked against
        the set of top level or global commands.
    type: list
  src:
    description:
      - The I(src) argument provides a path to the configuration file
        to load into the remote system.
      - The path can either be a full system path to the configuration
        file if the value starts with / or relative to the root of the
        implemented role or playbook.
      - This argument is mutually exclusive with the I(lines) and
        I(parents) arguments.
    type: path
  before:
    description:
      - The ordered set of commands to push on to the command stack if
        a change needs to be made.
      - This allows the playbook designer the opportunity to perform
        configuration commands prior to pushing any changes without
        affecting how the set of commands are matched against the system.
    type: list
  after:
    description:
      - The ordered set of commands to append to the end of the command
        stack if a change needs to be made.
      - Just like with I(before) this allows the playbook designer to
        append a set of commands to be executed after the command set.
    type: list
  match:
    description:
      - Instructs the module on the way to perform the matching of
        the set of commands against the current device config.
      - If match is set to I(line), commands are matched line by line.
      - If match is set to I(strict), command lines are matched with respect
        to position.
      - If match is set to I(exact), command lines must be an equal match.
      - Finally, if match is set to I(none), the module will not attempt to
        compare the source configuration with the running configuration on
        the remote device.
    type: str
    choices:
      - line
      - strict
      - exact
      - none
    default: line
  replace:
    description:
      - Instructs the module on the way to perform the configuration
        on the device.
      - If the replace argument is set to I(line) then the modified lines
        are pushed to the device in configuration mode.
      - If the replace argument is set to I(block) then the entire
        command block is pushed to the device in configuration mode if any
        line is not correct.
    type: str
    choices:
      - line
      - block
    default: line
  backup:
    description:
      - This argument will cause the module to create a full backup of
        the current C(running-config) from the remote device before any
        changes are made.
      - The backup file is written to the C(backup) folder in the playbook
        root directory or role root directory, if playbook is part of an
        ansible role. If the directory does not exist, it is created.
    type: bool
    default: 'no'
  running_config:
    description:
      - The module, by default, will connect to the remote device and
        retrieve the current running-config to use as a base for comparing
        against the contents of source.
      - There are times when it is not desirable to have the task get the
        current running-config for every task in a playbook.
      - The I(running_config) argument allows the implementer to pass in
        the configuration to use as the base config for comparison.
    type: str
    aliases: ['config']
  save_when:
    description:
      - When changes are made to the device running-configuration, the
        changes are not copied to non-volatile storage by default.
      - If the argument is set to I(always), then the running-config will
        always be copied to the startup-config and the I(modified) flag will
        always be set to C(True).
      - If the argument is set to I(modified), then the running-config
        will only be copied to the startup-config if it has changed since
        the last save to startup-config.
      - If the argument is set to I(never), the running-config will never be
        copied to the startup-config.
      - If the argument is set to I(changed), then the running-config
        will only be copied to the startup-config if the task has made a change.
    type: str
    choices:
      - always
      - never
      - modified
      - changed
    default: never
  diff_against:
    description:
      - When using the C(ansible-playbook --diff) command line argument
        the module can generate diffs against different sources.
      - When this option is configure as I(startup), the module will return
        the diff of the running-config against the startup-config.
      - When this option is configured as I(intended), the module will
        return the diff of the running-config against the configuration
        provided in the C(intended_config) argument.
      - When this option is configured as I(running), the module will
        return the before and after diff of the running-config with respect
        to any changes made to the device configuration.
    type: str
    choices:
      - startup
      - intended
      - running
    default: startup
  diff_ignore_lines:
    description:
      - Use this argument to specify one or more lines that should be
        ignored during the diff.
      - This is used for lines in the configuration that are automatically
        updated by the system.
      - This argument takes a list of regular expressions or exact line matches.
    type: list
  intended_config:
    description:
      - The C(intended_config) provides the master configuration that
        the node should conform to and is used to check the final
        running-config against.
      - This argument will not modify any settings on the remote device and
        is strictly used to check the compliance of the current device's
        configuration against.
      - When specifying this argument, the task should also modify the
        C(diff_against) value and set it to I(intended).
    type: str
  backup_options:
    description:
      - This is a dict object containing configurable options related to backup file path.
        The value of this option is read only when C(backup) is set to I(yes), if C(backup) is set
        to I(no) this option will be silently ignored.
    suboptions:
      filename:
        description:
          - The filename to be used to store the backup configuration. If the the filename
            is not given it will be generated based on the hostname, current time and date
            in format defined by <hostname>_config.<current-date>@<current-time>
        type: str
      dir_path:
        description:
          - This option provides the path ending with directory name in which the backup
            configuration file will be stored. If the directory does not exist it will be first
            created and the filename is either the value of C(filename) or default filename
            as described in C(filename) options description. If the path value is not given
            in that case a I(backup) directory will be created in the current working directory
            and backup configuration will be copied in C(filename) within I(backup) directory.
        type: path
    type: dict
    version_added: "2.8"
notes:
  - Abbreviated commands are NOT idempotent, see
    L(Network FAQ,../network/user_guide/faq.html#why-do-the-config-modules-always-return-changed-true-with-abbreviated-commands).
extends_documentation_fragment: f5
author:
  - Tim Rupp (@caphrim007)
'''

EXAMPLES = r'''
- name: configure top level configuration and save it
  bigip_imish_config:
    lines: bfd slow-timer 2000
    save_when: modified
    provider:
      user: admin
      password: secret
      server: lb.mydomain.com
  delegate_to: localhost

- name: diff the running-config against a provided config
  bigip_imish_config:
    diff_against: intended
    intended_config: "{{ lookup('file', 'master.cfg') }}"
    provider:
      user: admin
      password: secret
      server: lb.mydomain.com
  delegate_to: localhost

- name: Add config to a parent block
  bigip_imish_config:
    lines:
      - bgp graceful-restart restart-time 120
      - redistribute kernel route-map rhi
      - neighbor 10.10.10.11 remote-as 65000
      - neighbor 10.10.10.11 fall-over bfd
      - neighbor 10.10.10.11 remote-as 65000
      - neighbor 10.10.10.11 fall-over bfd
    parents: router bgp 64664
    match: exact
    provider:
      user: admin
      password: secret
      server: lb.mydomain.com
  delegate_to: localhost

- name: Remove an existing acl before writing it
  bigip_imish_config:
    lines:
      - access-list 10 permit 20.20.20.20
      - access-list 10 permit 20.20.20.21
      - access-list 10 deny any
    before: no access-list 10
    provider:
      user: admin
      password: secret
      server: lb.mydomain.com
  delegate_to: localhost

- name: for idempotency, use full-form commands
  bigip_imish_config:
    lines:
      # - desc My interface
      - description My Interface
    # parents: int ANYCAST-P2P-2
    parents: interface ANYCAST-P2P-2
    provider:
      user: admin
      password: secret
      server: lb.mydomain.com
  delegate_to: localhost

- name: configurable backup path
  bigip_imish_config:
    lines: bfd slow-timer 2000
    backup: yes
    provider:
      user: admin
      password: secret
      server: lb.mydomain.com
    backup_options:
      filename: backup.cfg
      dir_path: /home/user
  delegate_to: localhost
'''

RETURN = r'''
commands:
  description: The set of commands that will be pushed to the remote device
  returned: always
  type: list
  sample: ['interface ANYCAST-P2P-2', 'neighbor 20.20.20.21 remote-as 65000', 'neighbor 20.20.20.21 fall-over bfd']
updates:
  description: The set of commands that will be pushed to the remote device
  returned: always
  type: list
  sample: ['interface ANYCAST-P2P-2', 'neighbor 20.20.20.21 remote-as 65000', 'neighbor 20.20.20.21 fall-over bfd']
backup_path:
  description: The full path to the backup file
  returned: when backup is yes
  type: str
  sample: /playbooks/ansible/backup/bigip_imish_config.2016-07-16@22:28:34
'''


try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


import os
import tempfile

from ansible.module_utils.network.common.config import NetworkConfig, dumps
from ansible.module_utils.network.common.utils import to_list
from ansible.module_utils.basic import AnsibleModule

try:
    from library.module_utils.network.f5.bigip import F5RestClient
    from library.module_utils.network.f5.common import F5ModuleError
    from library.module_utils.network.f5.common import AnsibleF5Parameters
    from library.module_utils.network.f5.common import fq_name
    from library.module_utils.network.f5.common import f5_argument_spec
    from library.module_utils.network.f5.icontrol import upload_file
except ImportError:
    from ansible.module_utils.network.f5.bigip import F5RestClient
    from ansible.module_utils.network.f5.common import F5ModuleError
    from ansible.module_utils.network.f5.common import AnsibleF5Parameters
    from ansible.module_utils.network.f5.common import fq_name
    from ansible.module_utils.network.f5.common import f5_argument_spec
    from ansible.module_utils.network.f5.icontrol import upload_file


class Parameters(AnsibleF5Parameters):
    api_map = {

    }

    api_attributes = [

    ]

    returnables = [
        '__backup__',
        'commands',
        'updates'
    ]

    updatables = [

    ]


class ApiParameters(Parameters):
    pass


class ModuleParameters(Parameters):
    pass


class Changes(Parameters):
    def to_return(self):
        result = {}
        try:
            for returnable in self.returnables:
                result[returnable] = getattr(self, returnable)
            result = self._filter_params(result)
        except Exception:
            pass
        return result


class UsableChanges(Changes):
    pass


class ReportableChanges(Changes):
    pass


class Difference(object):
    def __init__(self, want, have=None):
        self.want = want
        self.have = have

    def compare(self, param):
        try:
            result = getattr(self, param)
            return result
        except AttributeError:
            return self.__default(param)

    def __default(self, param):
        attr1 = getattr(self.want, param)
        try:
            attr2 = getattr(self.have, param)
            if attr1 != attr2:
                return attr1
        except AttributeError:
            return attr1


class ModuleManager(object):
    def __init__(self, *args, **kwargs):
        self.module = kwargs.get('module', None)
        self.client = F5RestClient(**self.module.params)
        self.want = ModuleParameters(params=self.module.params)
        self.have = ApiParameters()
        self.changes = UsableChanges()

    def _set_changed_options(self):
        changed = {}
        for key in Parameters.returnables:
            if getattr(self.want, key) is not None:
                changed[key] = getattr(self.want, key)
        if changed:
            self.changes = UsableChanges(params=changed)

    def _update_changed_options(self):
        diff = Difference(self.want, self.have)
        updatables = Parameters.updatables
        changed = dict()
        for k in updatables:
            change = diff.compare(k)
            if change is None:
                continue
            else:
                if isinstance(change, dict):
                    changed.update(change)
                else:
                    changed[k] = change
        if changed:
            self.changes = UsableChanges(params=changed)
            return True
        return False

    def should_update(self):
        result = self._update_changed_options()
        if result:
            return True
        return False

    def exec_module(self):
        result = dict()
        changed = self.present()

        reportable = ReportableChanges(params=self.changes.to_return())
        changes = reportable.to_return()
        result.update(**changes)
        result.update(dict(changed=changed))
        return result

    def present(self):
        result = dict(changed=False)
        config = None
        contents = None

        if self.want.backup or (self.module._diff and self.want.diff_against == 'running'):
            contents = self.read_current_from_device()
            config = NetworkConfig(indent=1, contents=contents)
            if self.want.backup:
                # The backup file is created in the bigip_imish_config action plugin. Refer
                # to that if you have questions. The key below is removed by the action plugin.
                result['__backup__'] = contents

        if any((self.want.src, self.want.lines)):
            match = self.want.match
            replace = self.want.replace

            candidate = self.get_candidate()
            running = self.get_running_config(contents)

            response = self.get_diff(
                candidate=candidate,
                running=running,
                diff_match=match,
                diff_ignore_lines=self.want.diff_ignore_lines,
                path=self.want.parents,
                diff_replace=replace
            )

            config_diff = response['config_diff']

            if config_diff:
                commands = config_diff.split('\n')

                if self.want.before:
                    commands[:0] = self.want.before

                if self.want.after:
                    commands.extend(self.want.after)

                result['commands'] = commands
                result['updates'] = commands

                if not self.module.check_mode:
                    self.load_config(commands)

                result['changed'] = True

        running_config = self.want.running_config
        startup_config = None

        if self.want.save_when == 'always':
            self.save_config(result)
        elif self.want.save_when == 'modified':
            output = self.execute_show_commands(['show running-config', 'show startup-config'])

            running_config = NetworkConfig(indent=1, contents=output[0], ignore_lines=self.want.diff_ignore_lines)
            startup_config = NetworkConfig(indent=1, contents=output[1], ignore_lines=self.want.diff_ignore_lines)

            if running_config.sha1 != startup_config.sha1:
                self.save_config(result)
        elif self.want.save_when == 'changed' and result['changed']:
            self.save_on_device()

        if self.module._diff:
            if not running_config:
                output = self.execute_show_commands('show running-config')
                contents = output[0]
            else:
                contents = running_config

            # recreate the object in order to process diff_ignore_lines
            running_config = NetworkConfig(indent=1, contents=contents, ignore_lines=self.want.diff_ignore_lines)

            if self.want.diff_against == 'running':
                if self.module.check_mode:
                    self.module.warn("unable to perform diff against running-config due to check mode")
                    contents = None
                else:
                    contents = config.config_text

            elif self.want.diff_against == 'startup':
                if not startup_config:
                    output = self.execute_show_commands('show startup-config')
                    contents = output[0]
                else:
                    contents = startup_config.config_text

            elif self.want.diff_against == 'intended':
                contents = self.want.intended_config

            if contents is not None:
                base_config = NetworkConfig(indent=1, contents=contents, ignore_lines=self.want.diff_ignore_lines)

                if running_config.sha1 != base_config.sha1:
                    if self.want.diff_against == 'intended':
                        before = running_config
                        after = base_config
                    elif self.want.diff_against in ('startup', 'running'):
                        before = base_config
                        after = running_config

                    result.update({
                        'changed': True,
                        'diff': {'before': str(before), 'after': str(after)}
                    })
        self.changes.update(result)
        return result['changed']

    def load_config(self, commands):
        content = StringIO("\n".join(commands))

        file = tempfile.NamedTemporaryFile()
        name = os.path.basename(file.name)

        self.upload_file_to_device(content, name)
        self.load_config_on_device(name)
        self.remove_uploaded_file_from_device(name)

    def remove_uploaded_file_from_device(self, name):
        filepath = '/var/config/rest/downloads/{0}'.format(name)
        params = {
            "command": "run",
            "utilCmdArgs": filepath
        }
        uri = "https://{0}:{1}/mgmt/tm/util/unix-rm".format(
            self.client.provider['server'],
            self.client.provider['server_port']
        )
        resp = self.client.api.post(uri, json=params)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))
        if 'code' in response and response['code'] in [400, 403]:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

    def upload_file_to_device(self, content, name):
        url = 'https://{0}:{1}/mgmt/shared/file-transfer/uploads'.format(
            self.client.provider['server'],
            self.client.provider['server_port']
        )
        try:
            upload_file(self.client, url, content, name)
        except F5ModuleError:
            raise F5ModuleError(
                "Failed to upload the file."
            )

    def load_config_on_device(self, name):
        filepath = '/var/config/rest/downloads/{0}'.format(name)
        command = 'imish -r {0} -f {1}'.format(self.want.route_domain, filepath)

        params = {
            "command": "run",
            "utilCmdArgs": '-c "{0}"'.format(command)
        }
        uri = "https://{0}:{1}/mgmt/tm/util/bash".format(
            self.client.provider['server'],
            self.client.provider['server_port']
        )
        resp = self.client.api.post(uri, json=params)
        try:
            response = resp.json()
            if 'commandResult' in response:
                if 'Dynamic routing is not enabled' in response['commandResult']:
                    raise F5ModuleError(response['commandResult'])
        except ValueError as ex:
            raise F5ModuleError(str(ex))
        if 'code' in response and response['code'] in [400, 403]:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

    def read_current_from_device(self):
        command = 'imish -r {0} -e \\\"show running-config\\\"'.format(self.want.route_domain)

        params = {
            "command": "run",
            "utilCmdArgs": '-c "{0}"'.format(command)
        }
        uri = "https://{0}:{1}/mgmt/tm/util/bash".format(
            self.client.provider['server'],
            self.client.provider['server_port']
        )
        resp = self.client.api.post(uri, json=params)
        try:
            response = resp.json()
            if 'commandResult' in response:
                if 'Dynamic routing is not enabled' in response['commandResult']:
                    raise F5ModuleError(response['commandResult'])
        except ValueError as ex:
            raise F5ModuleError(str(ex))
        if 'code' in response and response['code'] in [400, 403]:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return response['commandResult']

    def save_on_device(self):
        command = 'imish -e write'
        params = {
            "command": "run",
            "utilCmdArgs": '-c "{0}"'.format(command)
        }
        uri = "https://{0}:{1}/mgmt/tm/util/bash".format(
            self.client.provider['server'],
            self.client.provider['server_port']
        )
        resp = self.client.api.post(uri, json=params)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))
        if 'code' in response and response['code'] in [400, 403]:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

    def get_diff(self, candidate=None, running=None, diff_match='line', diff_ignore_lines=None, path=None, diff_replace='line'):
        diff = {}

        # prepare candidate configuration
        candidate_obj = NetworkConfig(indent=1)
        candidate_obj.load(candidate)

        if running and diff_match != 'none' and diff_replace != 'config':
            # running configuration
            running_obj = NetworkConfig(indent=1, contents=running, ignore_lines=diff_ignore_lines)
            configdiffobjs = candidate_obj.difference(running_obj, path=path, match=diff_match, replace=diff_replace)
        else:
            configdiffobjs = candidate_obj.items

        diff['config_diff'] = dumps(configdiffobjs, 'commands') if configdiffobjs else ''
        return diff

    def get_running_config(self, config=None):
        contents = self.want.running_config
        if not contents:
            if config:
                contents = config
            else:
                contents = self.read_current_from_device()
        return contents

    def get_candidate(self):
        candidate = ''
        if self.want.src:
            candidate = self.want.src

        elif self.want.lines:
            candidate_obj = NetworkConfig(indent=1)
            parents = self.want.parents or list()
            candidate_obj.add(self.want.lines, parents=parents)
            candidate = dumps(candidate_obj, 'raw')
        return candidate

    def execute_show_commands(self, commands):
        body = []

        uri = "https://{0}:{1}/mgmt/tm/util/bash".format(
            self.client.provider['server'],
            self.client.provider['server_port']
        )
        for command in to_list(commands):
            command = 'imish -r {0} -e \\\"{1}\\\"'.format(self.want.route_domain, command)
            params = {
                "command": "run",
                "utilCmdArgs": '-c "{0}"'.format(command)
            }
            resp = self.client.api.post(uri, json=params)
            try:
                response = resp.json()
                if 'commandResult' in response:
                    if 'Dynamic routing is not enabled' in response['commandResult']:
                        raise F5ModuleError(response['commandResult'])
            except ValueError as ex:
                raise F5ModuleError(str(ex))
            if 'code' in response and response['code'] in [400, 403]:
                if 'message' in response:
                    raise F5ModuleError(response['message'])
                else:
                    raise F5ModuleError(resp.content)
            body.append(response['commandResult'])
        return body

    def save_config(self, result):
        result['changed'] = True
        if self.module.check_mode:
            self.module.warn(
                'Skipping command `copy running-config startup-config` '
                'due to check_mode.  Configuration not copied to '
                'non-volatile storage'
            )
            return
        self.save_on_device()


class ArgumentSpec(object):
    def __init__(self):
        self.supports_check_mode = True
        backup_spec = dict(
            filename=dict(),
            dir_path=dict(type='path')
        )
        argument_spec = dict(
            route_domain=dict(default=0),
            src=dict(type='path'),
            lines=dict(aliases=['commands'], type='list'),
            parents=dict(type='list'),

            before=dict(type='list'),
            after=dict(type='list'),

            match=dict(default='line', choices=['line', 'strict', 'exact', 'none']),
            replace=dict(default='line', choices=['line', 'block']),

            running_config=dict(aliases=['config']),
            intended_config=dict(),

            backup=dict(type='bool', default=False),
            backup_options=dict(type='dict', options=backup_spec),

            save_when=dict(choices=['always', 'never', 'modified', 'changed'], default='never'),

            diff_against=dict(choices=['running', 'startup', 'intended'], default='startup'),
            diff_ignore_lines=dict(type='list'),
        )
        self.argument_spec = {}
        self.argument_spec.update(f5_argument_spec)
        self.argument_spec.update(argument_spec)
        self.mutually_exclusive = [
            ('lines', 'src'),
            ('parents', 'src'),
        ]
        self.required_if = [
            ('match', 'strict', ['lines']),
            ('match', 'exact', ['lines']),
            ('replace', 'block', ['lines']),
            ('diff_against', 'intended', ['intended_config'])
        ]
        self.add_file_common_args = True


def main():
    spec = ArgumentSpec()

    module = AnsibleModule(
        argument_spec=spec.argument_spec,
        supports_check_mode=spec.supports_check_mode,
        mutually_exclusive=spec.mutually_exclusive,
        required_if=spec.required_if,
        add_file_common_args=spec.add_file_common_args,
    )

    client = F5RestClient(**module.params)

    try:
        mm = ModuleManager(module=module)
        results = mm.exec_module()
        module.exit_json(**results)
    except F5ModuleError as ex:
        module.fail_json(msg=str(ex))


if __name__ == '__main__':
    main()



# Copyright (c) 2019 Hetzner Cloud GmbH <info@hetzner-cloud.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

DOCUMENTATION = r"""
    name: hcloud
    plugin_type: inventory
    author:
      - Lukas Kaemmerling (@lkaemmerling)
    short_description: Ansible dynamic inventory plugin for the Hetzner Cloud.
    version_added: "2.8"
    requirements:
        - python >= 2.7
        - hcloud-python >= 1.0.0
    description:
        - Reads inventories from the Hetzner Cloud API.
        - Uses a YAML configuration file that ends with hcloud.(yml|yaml).
    extends_documentation_fragment:
        - constructed
    options:
        plugin:
            description: marks this as an instance of the "hcloud" plugin
            required: true
            choices: ["hcloud"]
        token:
            description: The Hetzner Cloud API Token.
            required: true
            env:
                - name: HCLOUD_TOKEN
        connect_with:
            description: Connect to the server using the value from this field.
            default: public_ipv4
            type: str
            choices:
                - public_ipv4
                - hostname
                - ipv4_dns_ptr
        locations:
          description: Populate inventory with instances in this location.
          default: []
          type: list
          required: false
        types:
          description: Populate inventory with instances with this type.
          default: []
          type: list
          required: false
        images:
          description: Populate inventory with instances with this image name, only available for system images.
          default: []
          type: list
          required: false
        label_selector:
          description: Populate inventory with instances with this label.
          default: ""
          type: str
          required: false
"""

EXAMPLES = r"""
# Minimal example. `HCLOUD_TOKEN` is exposed in environment.
plugin: hcloud

# Example with locations, types, groups and token
plugin: hcloud
token: foobar
locations:
  - nbg1
types:
  - cx11

# Group by a location with prefix e.g. "hcloud_location_nbg1"
# and image_os_flavor without prefix and separator e.g. "ubuntu"
# and status with prefix e.g. "server_status_running"
plugin: hcloud
keyed_groups:
  - key: location
    prefix: hcloud_location
  - key: image_os_flavor
    separator: ""
  - key: status
    prefix: server_status
"""

import os
from ansible.errors import AnsibleError
from ansible.module_utils._text import to_native
from ansible.plugins.inventory import BaseInventoryPlugin, Constructable
from ansible.release import __version__

try:
    from hcloud import hcloud
except ImportError:
    raise AnsibleError("The Hetzner Cloud dynamic inventory plugin requires hcloud-python.")


class InventoryModule(BaseInventoryPlugin, Constructable):
    NAME = "hcloud"

    def _configure_hcloud_client(self):
        self.api_token = self.get_option("token")
        if self.api_token is None:
            raise AnsibleError(
                "Please specify a token, via the option token or via environment variable HCLOUD_TOKEN")

        self.endpoint = os.getenv("HCLOUD_ENDPOINT") or "https://api.hetzner.cloud/v1"

        self.client = hcloud.Client(token=self.api_token,
                                    api_endpoint=self.endpoint,
                                    application_name="ansible-inventory",
                                    application_version=__version__)

    def _test_hcloud_token(self):
        try:
            # We test the API Token against the location API, because this is the API with the smallest result
            # and not controllable from the customer.
            self.client.locations.get_all()
        except hcloud.APIException:
            raise AnsibleError("Invalid Hetzner Cloud API Token.")

    def _get_servers(self):
        if len(self.get_option("label_selector")) > 0:
            self.servers = self.client.servers.get_all(label_selector=self.get_option("label_selector"))
        else:
            self.servers = self.client.servers.get_all()

    def _filter_servers(self):
        if self.get_option("locations"):
            tmp = []
            for server in self.servers:
                if server.datacenter.location.name in self.get_option("locations"):
                    tmp.append(server)
            self.servers = tmp

        if self.get_option("types"):
            tmp = []
            for server in self.servers:
                if server.server_type.name in self.get_option("types"):
                    tmp.append(server)
            self.servers = tmp

        if self.get_option("images"):
            tmp = []
            for server in self.servers:
                if server.image is not None and server.image.os_flavor in self.get_option("images"):
                    tmp.append(server)
            self.servers = tmp

    def _set_server_attributes(self, server):
        self.inventory.set_variable(server.name, "id", to_native(server.id))
        self.inventory.set_variable(server.name, "name", to_native(server.name))
        self.inventory.set_variable(server.name, "status", to_native(server.status))
        self.inventory.set_variable(server.name, "type", to_native(server.server_type.name))

        # Network
        self.inventory.set_variable(server.name, "ipv4", to_native(server.public_net.ipv4.ip))
        self.inventory.set_variable(server.name, "ipv6_network", to_native(server.public_net.ipv6.network))
        self.inventory.set_variable(server.name, "ipv6_network_mask", to_native(server.public_net.ipv6.network_mask))

        if self.get_option("connect_with") == "public_ipv4":
            self.inventory.set_variable(server.name, "ansible_host", to_native(server.public_net.ipv4.ip))
        elif self.get_option("connect_with") == "hostname":
            self.inventory.set_variable(server.name, "ansible_host", to_native(server.name))
        elif self.get_option("connect_with") == "ipv4_dns_ptr":
            self.inventory.set_variable(server.name, "ansible_host", to_native(server.public_net.ipv4.dns_ptr))

        # Server Type
        self.inventory.set_variable(server.name, "server_type", to_native(server.image.name))

        # Datacenter
        self.inventory.set_variable(server.name, "datacenter", to_native(server.datacenter.name))
        self.inventory.set_variable(server.name, "location", to_native(server.datacenter.location.name))

        # Image
        self.inventory.set_variable(server.name, "image_id", to_native(server.image.id))
        self.inventory.set_variable(server.name, "image_name", to_native(server.image.name))
        self.inventory.set_variable(server.name, "image_os_flavor", to_native(server.image.os_flavor))

    def verify_file(self, path):
        """Return the possibly of a file being consumable by this plugin."""
        return (
            super(InventoryModule, self).verify_file(path) and
            path.endswith((self.NAME + ".yaml", self.NAME + ".yml"))
        )

    def parse(self, inventory, loader, path, cache=True):
        super(InventoryModule, self).parse(inventory, loader, path, cache)
        self._read_config_data(path)
        self._configure_hcloud_client()
        self._test_hcloud_token()
        self._get_servers()
        self._filter_servers()

        # Add a top group 'hcloud'
        self.inventory.add_group(group="hcloud")

        for server in self.servers:
            self.inventory.add_host(server.name, group="hcloud")
            self._set_server_attributes(server)

            # Use constructed if applicable
            strict = self.get_option('strict')

            # Composed variables
            self._set_composite_vars(self.get_option('compose'), {}, server.name, strict=strict)

            # Complex groups based on jinja2 conditionals, hosts that meet the conditional are added to group
            self._add_host_to_composed_groups(self.get_option('groups'), {}, server.name, strict=strict)

            # Create groups based on variable values and add the corresponding hosts to it
            self._add_host_to_keyed_groups(self.get_option('keyed_groups'), {}, server.name, strict=strict)
